#####################################################################################################
#####################################################################################################
### ABIOTIC LAKE VARIATION AND ITS POSSIBLE EFFECT ON TRAIT DIVERGENCE IN SCULPIN AND STICKLEBACK ###
### Code used for Roesti et al. (Ecology Letters) ###################################################
#####################################################################################################
#####################################################################################################

rm(list = ls(all = TRUE))

# Set path to the downloaded dryad folder containing all codes and data for Roesti et al.
setwd("XXXX")

# Load packages
library(missMDA)
library(FactoMineR)
library(factoextra)
library(car)

# Load data
abiotic.data <- read.csv("data/abiotic.lake.variation.csv", header = TRUE, stringsAsFactor = FALSE)
gac.popMeans <- read.csv("data/morphology.population.means_stickleback.csv", header = TRUE, stringsAsFactor = FALSE) # stickleback population trait means and morphological PC values as obtained from running "R.code_morphological.traits.r"
cas.popMeans <- read.csv("data/morphology.population.means_sculpin.csv", header = TRUE, stringsAsFactor = FALSE) # sculpin population trait means and morphological PC values as obtained from running "R.code_morphological.traits.r"


#############################################################################
## Analysis of abiotic variation among study lakes ##########################
## Q1: Do allopatric and sympatric lakes differ in abiotic variation? #######
## Q2: Is abiotic variation among lakes explained by geographic proximity? ##
#############################################################################

## PCA for abiotic variation (10 variables) and for geographic coordinates of study lakes

# Impute missing data to allow running a PCA
nb <- estim_ncpPCA(abiotic.data[, c(5:14)], ncp.min = 0, ncp.max = 5)
abiotic.data.imputed <- imputePCA(abiotic.data[, c(5:14)], ncp = nb$ncp)
abiotic.data.imputed <- data.frame(cbind(abiotic.data[, c(1:4)], abiotic.data.imputed$completeObs))

# Round imputed data values to same resolution as observed values
abiotic.data.imputed[,8] <- round(as.numeric(as.character(abiotic.data.imputed[,8])),1)
abiotic.data.imputed[,9] <- round(as.numeric(as.character(abiotic.data.imputed[,9])),1)
abiotic.data.imputed[,11] <- round(as.numeric(as.character(abiotic.data.imputed[,11])),2)
abiotic.data.imputed[,12] <- round(as.numeric(as.character(abiotic.data.imputed[,12])),2)
abiotic.data.imputed[,13] <- round(as.numeric(as.character(abiotic.data.imputed[,13])),2)
row.names(abiotic.data.imputed) <- abiotic.data.imputed$lake
abiotic.data.imputed # looking good

# PCA based on the correlation matrix of abiotic lake characteristics; add abiotic.PC1 and abiotic.PC2 to data frame
abiotic.pca <- prcomp(abiotic.data.imputed[, -c(1:4)], scale. = T)
summary(abiotic.pca) # how much of the total abiotic variation do the first two pc-axes explain?
abiotic.data <- cbind(abiotic.data, abiotic.pca$x[, 1], abiotic.pca$x[,2])

abiotic.data[, dim(abiotic.data)[2]] <- abiotic.data[, dim(abiotic.data)[2]]*(-1) # invert PC2 for visualization
names(abiotic.data)[15] <- "abiotic.pc1"
names(abiotic.data)[16] <- "abiotic.pc2"

# PCA based on the correlation matrix of lat + long lake coordinates
latlong.pca <- prcomp(abiotic.data[, c(3,4)], scale. = T)
latlong.pc1 <- latlong.pca$x[,1]
abiotic.data <- cbind(abiotic.data, latlong.pc1)


## Visualize results (Figure 1 of the Supplementary Analysis 1)

# Prepare data for visualization
abiotic.data$symbol.shape <- "N"
abiotic.data$symbol.shape[which(abiotic.data$lake.type == "allopatric.sculpin")] <- "B"
abiotic.data$symbol.shape[which(abiotic.data$lake.type == "allopatric.stickleback")] <- "C"
abiotic.data$symbol.shape[which(abiotic.data$lake.type == "sympatric")] <- "A"

# Visualize abiotic PC1 and PC2 of each study lake
p <- suppressWarnings(ggplot(abiotic.data, aes(x = abiotic.pc1, y = abiotic.pc2, color = lake.type, shape = symbol.shape, size = 2)) +
     ylab("PC2 (17.3%)") +
     xlab("PC1 (42.6%)") +
     geom_point() +
     guides(color = "none", shape = "none", size = "none") +
     theme_minimal())
p

# Loadings of abiotic variables on PC1 and PC2 (note that PC2 was inverted in the plot above)
fviz_pca_var(abiotic.pca,
    col.var = "contrib",
    gradient.cols = c("#E7B800", "#FC4E07"),
    repel = TRUE
    )

# Visualize the association between abiotic PC1 and geography (Figure 1 of the Supplementary Analysis 1)
p <- suppressWarnings(ggplot(abiotic.data, aes(x = latlong.pc1, y = abiotic.pc1, color = lake.type, shape = symbol.shape, size = 1.5)) +
     ylab("Abiotic environment PC1 (42.6%)") +
     xlab("Geographic position (PC1 of lat + long)")) +
     geom_point() +
     guides(color = "none", shape = "none", size = "none") +
     theme_minimal()
p


## Calculate summary statistics (mean, SE) of abiotic variables per lake type

summary.stats <- NULL

for(i in 5:14){
    sc <- subset(abiotic.data, abiotic.data$lake.type == "allopatric.sculpin")[, i]
    sc <- sc[!is.na(sc)]
    st <- subset(abiotic.data, abiotic.data$lake.type == "allopatric.stickleback")[, i]
    st <- st[!is.na(st)]
    sy <- subset(abiotic.data, abiotic.data$lake.type == "sympatric")[, i]
    sy <- sy[!is.na(sy)]
    
    sc.mean <- round(mean(sc),2)
    sc.SE <- round(sd(sc)/sqrt(length(sc)),2)
    
    st.mean <- round(mean(st),2)
    st.SE <- round(sd(st)/sqrt(length(st)),2)
    
    sy.mean <- round(mean(sy),2)
    sy.SE <- round(sd(sy)/sqrt(length(sy)),2)
    
    info <- cbind(names(data)[i], sc.mean, sc.SE, st.mean, st.SE, sy.mean, sy.SE)
    summary.stats <- rbind(summary.stats, info)
}

summary.stats <- data.frame(t(summary.stats))
names(summary.stats) <- names(abiotic.data)[c(5:14)]
summary.stats


## Testing for a difference in abiotic variables between allopatric lakes versus sympatric lakes

iter <- 10000 # number of resampling iterations for calculating P-values
abiotic.data <- abiotic.data[, -c(dim(abiotic.data)[2]-1, dim(abiotic.data)[2])] # delete the "symbol.shape" column, which is no longer needed

# Allopatric stickleback lakes vs. sympatric lakes
abiotic.data.sub <- subset(abiotic.data, abiotic.data$lake.type != "allopatric.sculpin")
signficiance.stats <- NULL
for(t in 5:length(abiotic.data.sub[1,])){
    sym.mean <- mean(subset(abiotic.data.sub, abiotic.data.sub$lake.type == "sympatric")[,t], na.rm = T)
    allo.mean <- mean(subset(abiotic.data.sub, abiotic.data.sub$lake.type == "allopatric.stickleback")[,t], na.rm = T)
    obs.diff <- sym.mean-allo.mean

    cnt <- 0
    for(i in 1:iter){
        x <- abiotic.data.sub
        x[, t] <- abiotic.data.sub[sample(1:length(abiotic.data.sub[,t])), t]
        sym.mean <- mean(subset(x, x$lake.type == "sympatric")[,t], na.rm = T)
        allo.mean <- mean(subset(x, x$lake.type == "allopatric.stickleback")[,t], na.rm = T)
        if(is.na(allo.mean) == "FALSE" && is.na(sym.mean) == "FALSE"){
            sim.diff <- sym.mean-allo.mean
            if(abs(sim.diff) >= abs(obs.diff)){cnt <- cnt+1}  # "abs()" makes this a two-tailed test
            }
    }
    p.value <- round(as.numeric(cnt/iter), digits = 3)
    inf <- cbind(names(abiotic.data.sub)[t], p.value)
    signficiance.stats <- rbind(signficiance.stats, inf)
}
a <- cbind.data.frame("sym-allo.gac", signficiance.stats)
names(a)[1] <- "lake.type.comparison"
names(a)[2] <- "abiotic.variable"

# Allopatric sculpin lakes vs. sympatric lakes
abiotic.data.sub <- subset(abiotic.data, abiotic.data$lake.type != "allopatric.stickleback")
signficiance.stats <- NULL
for(t in 5:length(abiotic.data.sub[1,])){
    sym.mean <- mean(subset(abiotic.data.sub, abiotic.data.sub$lake.type == "sympatric")[,t], na.rm = T)
    allo.mean <- mean(subset(abiotic.data.sub, abiotic.data.sub$lake.type == "allopatric.sculpin")[,t], na.rm = T)
    obs.diff <- sym.mean-allo.mean

    cnt <- 0
    for(i in 1:iter){
        x <- abiotic.data.sub
        x[, t] <- abiotic.data.sub[sample(1:length(abiotic.data.sub[,t])),t]
        sym.mean <- mean(subset(x, x$lake.type == "sympatric")[,t], na.rm = T)
        allo.mean <- mean(subset(x, x$lake.type == "allopatric.sculpin")[,t], na.rm = T)
        if(is.na(allo.mean) == "FALSE" && is.na(sym.mean) == "FALSE"){
            sim.diff <- sym.mean-allo.mean
            if(abs(sim.diff) >= abs(obs.diff)){cnt <- cnt+1}
            }
    }
    p.value <- round(as.numeric(cnt/iter), digits = 3)
    inf <- cbind(names(abiotic.data.sub)[t], p.value)
    signficiance.stats <- rbind(signficiance.stats, inf)
}
b <- cbind.data.frame("sym-allo.cas", signficiance.stats)
names(b)[1] <- "lake.type.comparison"
names(b)[2] <- "abiotic.variable"
significance.statistics.complete <- rbind(a, b)
significance.statistics.complete


###########################################################################################################################
## Testing whether abiotic variation can explain morphological differences in sculpin and stickleback between lake types ##
## Supplementary Analysis 2 ###############################################################################################
###########################################################################################################################


## Stickleback

names(abiotic.data)[1] <- "population"
gac.data <- merge(abiotic.data, gac.popMeans[,-2], by = "population")

# Testing for an effect of "abiotic.habitat.pc1" on "foraging.pc1"
model <- lm(foraging.pc1 ~ lake.type + abiotic.pc1, data = gac.data)
Anova(model, type = "III")

# Testing for an effect of "abiotic.habitat.pc2" on "foraging.pc1"
model <- lm(foraging.pc1 ~ lake.type + abiotic.pc2, data = gac.data)
Anova(model, type = "III")

# Testing for effects of "abiotic.habitat.pc1" and/or "abiotic.habitat.pc2" on "foraging.pc1"
model <- lm(foraging.pc1 ~ lake.type + abiotic.pc1 + abiotic.pc2, data = gac.data)
Anova(model, type = "III")

# Testing for an effect of "abiotic.habitat.pc1" on "antipredator.pc1"
model <- lm(antipredator.pc1 ~ lake.type + abiotic.pc1, data = gac.data)
Anova(model, type = "III")

# Testing for an effect of "abiotic.habitat.pc2" on "antipredator.pc1"
model <- lm(antipredator.pc1 ~ lake.type + abiotic.pc2, data = gac.data)
Anova(model, type = "III")

# Testing for effects of "abiotic.habitat.pc1" and/or "abiotic.habitat.pc2" on "antipredator.pc1"
model <- lm(antipredator.pc1 ~ lake.type + abiotic.pc1 + abiotic.pc2 + lake.type, data = gac.data)
Anova(model, type = "III")


## Sculpin

cas.data <- merge(abiotic.data, cas.popMeans[,-2], by = "population")

# Testing for an effect of "abiotic.habitat.pc1" on "foraging.pc2"
model <- lm(foraging.pc2 ~ lake.type + abiotic.pc1, data = cas.data)
Anova(model, type = "III")

# Testing for an effect of "abiotic.habitat.pc2" on "foraging.pc2"
model <- lm(foraging.pc2 ~ lake.type + abiotic.pc2, data = cas.data)
Anova(model, type = "III")

# Testing for effects of "abiotic.habitat.pc1" and/or "abiotic.habitat.pc2" on "foraging.pc2"
model <- lm(foraging.pc2 ~ lake.type + abiotic.pc1 + abiotic.pc2 + lake.type, data = cas.data)
Anova(model, type = "III")

# Testing for an effect of "abiotic.habitat.pc1" on "antipredator.pc1"
model <- lm(antipredator.pc1 ~ lake.type + abiotic.pc1, data = cas.data)
Anova(model, type = "III")
# compare to:
model <- lm(antipredator.pc1 ~ lake.type, data = cas.data)
Anova(model, type = "III")

# Testing for an effect of "abiotic.habitat.pc2" on "antipredator.pc1"
model <- lm(antipredator.pc1 ~ lake.type + abiotic.pc2, data = cas.data)
Anova(model, type = "III")
# Yet, there is no correlation within each lake type:
allo <- subset(cas.data, cas.data$lake.type == "allopatric.sculpin")
sym <- subset(cas.data, cas.data$lake.type == "sympatric")
cor.test(allo$antipredator.pc1, allo$abiotic.pc2)
cor.test(sym$antipredator.pc1, sym$abiotic.pc2)

# Testing for effects of "abiotic.habitat.pc1" and/or "abiotic.habitat.pc2" on "antipredator.pc1"
model <- lm(antipredator.pc1 ~ lake.type + abiotic.pc1 + abiotic.pc2 + lake.type, data = cas.data)
Anova(model, type = "III")
