#####################################################
#####################################################
### STICKLEBACK BODY SHAPE ##########################
### Code used for Roesti et al. (Ecology Letters) ###
#####################################################
#####################################################

rm(list = ls(all = TRUE))

# Set path to the downloaded dryad folder containing all codes and data for Roesti et al.
setwd("XXXX")

# Load packages
library(geomorph)
library(MASS)
library(ggplot2)

# Load data
landmark.data <- readland.tps("data/landmark.data_stickleback.tps", specID = "ID") # scaled and unbent landmark data
landmark.matrix <- read.csv("data/landmark.matrix_stickleback.csv", header = FALSE, stringsAsFactor = FALSE) # landmark matrix for visualizing shape differences


######################################################################
## 1) Prepare landmark data for analysis, including size correction ##
######################################################################

# Create individual-level grouping variables by parsing specimen label
temp <- strsplit(names(landmark.data[1,1,]), "_")
IDs <- factor(unlist(lapply(lapply(temp, tail, 4L), head, 1)))
population <- factor(gsub(" ", "", unlist(lapply(temp, tail, 1L))))
lake.type <- factor(unlist(lapply(lapply(temp, tail, 3L), head, 1)))

# Sample sizes
table(population)
length(population)

# Procrustes superimposition
y <- gpagen(landmark.data)
plotAllSpecimens(y$coords) # plot procrustes coordinates, looking good

# Create geomorph data frame for analysis
geom.data <- geomorph.data.frame(y, ID = IDs, lake.type = lake.type, population = population)

# Size correction, using centroid size as 'size'
model <- procD.lm(coords ~ Csize, data = geom.data, iter = 0, print.progress = FALSE)


#############################################
## 2) Analyze and visualize the shape data ##
#############################################

####################################################################################################
# 2.1) LDA and obtaining summary and significance statistics for the difference between lake types #
####################################################################################################

# LDA of residuals, grouping by population
LDA <- suppressWarnings(lda(grouping = population, x = model$residuals))
scores <- predict(LDA, model$residuals)$x
LD1 <- tapply(scores[,1], population, mean)

# Linear combination of predictor variables used to form the LDA decision rule
predictors <- data.frame(cbind(row.names(LDA$scaling), LDA$scaling[,1]))
predictors[,2] <- as.numeric(as.character(predictors[,2]))
names(predictors) <- c("landmark.coordinate", "LD1.coefficients")

## Testing for a shape difference between lake types: calculatate grand means of LD1 per lake type, non-parametric P-values and 95% CI
lake.type <- c("sym", "allo", "sym", "allo", "allo", "allo", "allo", "allo", "sym", "sym", "allo", "sym", "sym", "sym", "sym", "sym", "allo", "allo")
shape.data <- data.frame(cbind(row.names(LD1), lake.type, LD1))
shape.data$LD1 <- as.numeric(shape.data$LD1)
names(shape.data)[1] <- "population"

# Allopatric and sympatric LD1 grand means
sym <- shape.data[which(shape.data$lake.type == "sym"), "LD1"]
sym.mean.obs <- mean(sym)
allo <- shape.data[which(shape.data$lake.type == "allo"), "LD1"]
allo.mean.obs <- mean(allo)

# Set parameters for calculating non-parametric P-value and 95% CI
alp <- 0.05 # alpha-level
iter <- 10000 # number of resampling iterations

# P-value
cnt <- 0
for(i in 1:iter){
    x <- shape.data
    x[,3] <- shape.data[sample(1:length(shape.data[,3])), "LD1"]
    allo.mean.rand <- mean(subset(x, x$lake.type == "allo")[, "LD1"])
    sym.mean.rand <- mean(subset(x, x$lake.type == "sym")[, "LD1"])
    if(abs(allo.mean.rand-sym.mean.rand) >= abs(allo.mean.obs-sym.mean.obs)){cnt <- cnt + 1}
}
Pvalue <- round(as.numeric(cnt/iter), digits = 3)
Pvalue

# Sympatric 95% CI
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(sym, length(sym), replace = T))
}
low.CI.sym <- sort(rdist)[alp/2*iter+1]
upper.CI.sym <- sort(rdist)[(1-(alp/2))*iter]

# Allopatric 95% CI
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(allo, length(allo), replace = T))
}
low.CI.allo <- sort(rdist)[alp/2*iter+1]
upper.CI.allo <- sort(rdist)[(1-(alp/2))*iter]


######################################################
# 2.2) Visualizing LDA results and shape differences #
######################################################

## Visualize shape LD1 (raw version of Figure 4a)
p <- ggplot(shape.data, aes(x = lake.type, y = LD1)) +
     labs(y = "Stickleback body shape LD1", x = "Lake Type") +
     geom_jitter(position = position_jitter(0.05), shape = 1, cex = 3, stroke = 1.25) +
     geom_segment(aes(x = 1.25, y = low.CI.allo, xend = 1.25, yend = upper.CI.allo), colour = "grey", size = 0.75) +
     geom_segment(aes(x = 1.75, y = low.CI.sym, xend = 1.75, yend = upper.CI.sym), colour = "grey", size = 0.75) +
     geom_point(aes(x = 1.25, y = allo.mean.obs), shape = 19, colour = "grey", size = 6) +
     geom_point(aes(x = 1.75, y = sym.mean.obs), shape = 19, colour = "grey", size = 6) +
     theme_minimal()
p

## Visualize the relative shape difference of two random individuals of the extreme populations (raw version of Figure 4a)
symp.ind <- grep("Gac0781", row.names(scores))
allo.ind <- grep("Gac0764", row.names(scores))
par(mfrow = c(2,1), mar = c(0,0,1,0))
plotRefToTarget(y$coords[,,allo.ind], y$coords[,,symp.ind])
plotRefToTarget(y$coords[,,symp.ind], y$coords[,,allo.ind])

## Visualize the average shape difference between sympatric and allopatric populations
# Collect size-corrected landmark coordinates for all individuals
a <- NULL
for(i in 1:21){
    tempA <- data.frame(t(geom.data$coords[i,,]))[,1] + model$residuals[,(i*2-1)]
    tempB <- data.frame(t(geom.data$coords[i,,]))[,2] + model$residuals[,i*2]
    temp <- cbind.data.frame(tempA, tempB)
    names(temp)[1] <- paste0(i, ".X")
    names(temp)[2] <- paste0(i, ".Y")
    if(i == 1){a <- temp}
    if(i > 1)a <- cbind(a, temp)
}

# Allopatric grand mean of size-corrected landmarks
allopatric <- c("Black", "Bullocks", "Cranby", "Hoggan", "Kirk", "Klein", "Mike", "Trout", "Weston")
allo.popMeans <- NULL
for(t in 1:length(allopatric)){
    temp <- colMeans(a[grep(allopatric[t], row.names(a)), ])
    allo.popMeans <- rbind(allo.popMeans, temp)
}
allo <- as.matrix(colMeans(allo.popMeans))

# Sympatric grand mean of size-corrected landmarks
sympatric <- c("Ambrose", "Blackwater", "McNair", "Merrill", "Morton", "North", "Pachena", "Paq", "Roselle")
symp.popMeans <- NULL
for(t in 1:length(sympatric)){
    temp <- colMeans(a[grep(sympatric[t], row.names(a)), ])
    symp.popMeans <- rbind(symp.popMeans, temp)
}
symp <- as.matrix(colMeans(symp.popMeans))

# Visualize grand mean difference (magnified x 2; raw version of the SupplMat Figure)
allo.tps <- NULL
symp.tps <- NULL
for(i in 1:21){
    allo.tps <- rbind(allo.tps, allo[c(i*2-1,i*2),])
    symp.tps <- rbind(symp.tps, symp[c(i*2-1,i*2),])
}
plotRefToTarget(symp.tps, allo.tps, method = "points", mag = 2, gridPars = gridPar(pt.size = 0.75), links = landmark.matrix) # sympatric = gray, allopatric = black


########################################################################################################################################
########################################################################################################################################


#####################################################
#####################################################
### SCLUPIN BODY SHAPE ##############################
### Code used for Roesti et al. (Ecology Letters) ###
#####################################################
#####################################################

rm(list = ls(all = TRUE))

# Set path to the downloaded dryad folder containing all codes and data for Roesti et al.
setwd("XXXX")

# Load packages
library(geomorph)
library(MASS)
library(ggplot2)

# Load data
landmark.data <- readland.tps("data/landmark.data_sculpin.tps", specID = "ID")  # scaled landmark data
landmark.matrix <- read.csv("data/landmark.matrix_sculpin.csv", header = FALSE, stringsAsFactor = FALSE) # landmark matrix for visualizing shape differences


######################################################################
## 1) Prepare landmark data for analysis, including size correction ##
######################################################################

# Create individual-level grouping variables by parsing specimen label
temp <- strsplit(names(landmark.data[1,1,]), "_")
IDs <- factor(unlist(lapply(lapply(temp, tail, 3L), head, 1)))
population <- factor(gsub(" ", "", unlist(lapply(temp, tail, n = 1L))))
lake.type <- factor(unlist(lapply(lapply(temp, tail, 2L), head, 1)))

# Sample sizes
table(population)
length(population)

# Procrustes superimposition
y <- gpagen(landmark.data)
plotAllSpecimens(y$coords) # plot procrustes coordinates, looking good

# Create geomorph data frame for analysis
geom.data <- geomorph.data.frame(y, ID = IDs, lake.type = lake.type, population = population)

# Size correction, using centroid size as 'size'
model <- procD.lm(coords ~ Csize, data = geom.data, iter = 0, print.progress = FALSE)


#############################################
## 2) Analyze and visualize the shape data ##
#############################################

####################################################################################################
# 2.1) LDA and obtaining summary and significance statistics for the difference between lake types #
####################################################################################################

# LDA of residuals, grouping by population
LDA <- suppressWarnings(lda(grouping = population, x = model$residuals))
scores <- predict(LDA, model$residuals)$x
LD1 <- tapply(scores[,1], population, mean)

# Linear combination of predictor variables used to form the LDA decision rule
predictors <- data.frame(cbind(row.names(LDA$scaling), LDA$scaling[,1]))
predictors[,2] <- as.numeric(as.character(predictors[,2]))
names(predictors) <- c("landmark.coordinate", "LD1.coefficients")

## Testing for a shape difference between lake types: calculatate grand means of LD1 per lake type, non-parametric P-values and 95% CI
lake.type <- c("sym", "allo", "allo", "sym", "allo", "allo", "allo", "sym", "sym", "sym", "allo", "sym", "sym", "sym", "sym", "allo", "allo")
shape.data <- data.frame(cbind(row.names(LD1), lake.type, LD1))
shape.data$LD1 <- as.numeric(shape.data$LD1)
names(shape.data)[1] <- "population"

# Allopatric and sympatric LD1 grand means
sym <- shape.data[which(shape.data$lake.type == "sym"), "LD1"]
sym.mean.obs <- mean(sym)
allo <- shape.data[which(shape.data$lake.type == "allo"), "LD1"]
allo.mean.obs <- mean(allo)

# Set parameters for calculating non-parametric P-value and 95% CI
alp <- 0.05 # alpha-level
iter <- 10000 # number of resampling iterations

# P-value
cnt <- 0
for(i in 1:iter){
    x <- shape.data
    x[,3] <- shape.data[sample(1:length(shape.data[,3])), "LD1"]
    allo.mean.rand <- mean(subset(x, x$lake.type == "allo")[, "LD1"])
    sym.mean.rand <- mean(subset(x, x$lake.type == "sym")[, "LD1"])
    if(abs(allo.mean.rand-sym.mean.rand) >= abs(allo.mean.obs-sym.mean.obs)){cnt <- cnt + 1}
}
Pvalue <- round(as.numeric(cnt/iter), digits = 3)
Pvalue

# Sympatric 95% CI
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(sym, length(sym), replace = T))
}
low.CI.sym <- sort(rdist)[alp/2*iter+1]
upper.CI.sym <- sort(rdist)[(1-(alp/2))*iter]

# Allopatric 95% CI
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(allo, length(allo), replace = T))
}
low.CI.allo <- sort(rdist)[alp/2*iter+1]
upper.CI.allo <- sort(rdist)[(1-(alp/2))*iter]


######################################################
# 2.2) Visualizing LDA results and shape differences #
######################################################

## Visualize shape LD1 (raw version of Figure 4b)
p <- ggplot(shape.data, aes(x = lake.type, y = LD1)) +
     labs(y = "Sculpin body shape LD1", x = "Lake Type") +
     geom_jitter(position = position_jitter(0.05), shape = 1, cex = 3, stroke = 1.25) +
     geom_segment(aes(x = 1.25, y = low.CI.allo, xend = 1.25, yend = upper.CI.allo), colour = "grey", size = 0.75) +
     geom_segment(aes(x = 1.75, y = low.CI.sym, xend = 1.75, yend = upper.CI.sym), colour = "grey", size = 0.75) +
     geom_point(aes(x = 1.25, y = allo.mean.obs), shape = 19, colour = "grey", size = 6) +
     geom_point(aes(x = 1.75, y = sym.mean.obs), shape = 19, colour = "grey", size = 6) +
     theme_minimal()
p

## Visualize the relative shape difference of two random individuals of the extreme populations (raw version of Figure 4b)
symp.ind <- grep("Cas273", row.names(scores))
allo.ind <- grep("Cas081", row.names(scores))
par(mfrow = c(2,1), mar = c(0,0,1,0))
plotRefToTarget(y$coords[,,allo.ind], y$coords[,,symp.ind])
plotRefToTarget(y$coords[,,symp.ind], y$coords[,,allo.ind])

## Visualize the average shape difference between sympatric and allopatric populations
# Collect size-corrected landmark coordinates for all individuals
a <- NULL
for(i in 1:19){
    tempA <- data.frame(t(geom.data$coords[i,,]))[,1] + model$residuals[,(i*2-1)]
    tempB <- data.frame(t(geom.data$coords[i,,]))[,2] + model$residuals[,i*2]
    temp <- cbind.data.frame(tempA, tempB)
    names(temp)[1] <- paste0(i, ".X")
    names(temp)[2] <- paste0(i, ".Y")
    if(i == 1){a <- temp}
    if(i > 1)a <- cbind(a, temp)
}

# Allopatric grand mean of size-corrected landmarks
allopatric <- c("Antler", "Beavertail", "Drum", "Keogh", "Kissinger", "Mukwilla", "Tuck", "Twin")
allo.popMeans <- NULL
for(t in 1:length(allopatric)){
    temp <- colMeans(a[grep(allopatric[t], row.names(a)), ])
    allo.popMeans <- rbind(allo.popMeans, temp)
}
allo <- as.matrix(colMeans(allo.popMeans))

# Sympatric grand mean of size-corrected landmarks
sympatric <- c("Ambrose", "Blackwater", "McNair", "Merrill", "Morton", "North", "Pachena", "Paq", "Roselle")
symp.popMeans <- NULL
for(t in 1:length(sympatric)){
    temp <- colMeans(a[grep(sympatric[t], row.names(a)), ])
    symp.popMeans <- rbind(symp.popMeans, temp)
}
symp <- as.matrix(colMeans(symp.popMeans))

# Visualize grand mean difference (magnified x 2; raw version of the SupplMat Figure)
allo.tps <- NULL
symp.tps <- NULL
for(i in 1:19){
    allo.tps <- rbind(allo.tps, allo[c(i*2-1,i*2),])
    symp.tps <- rbind(symp.tps, symp[c(i*2-1,i*2),])
}
plotRefToTarget(symp.tps, allo.tps, method = "points", mag = 2, gridPars = gridPar(pt.size = 0.75), links = landmark.matrix) # sympatric = gray, allopatric = black
