#####################################################
#####################################################
### STICKLEBACK MORPHOLOGICAL TRAIT ANALYSES ########
### Code used for Roesti et al. (Ecology Letters) ###
#####################################################
#####################################################

rm(list = ls(all = TRUE))

# Set path to the downloaded dryad folder containing all codes and data for Roesti et al.
setwd("XXXX")

# Load packages
library(MASS)
library(FactoMineR)
library(factoextra)
library(corrplot)
library(car)
library(ppcor)
library(emmeans)
library(ggplot2)
library(lme4)
library(dplyr)
library(Hmisc)
library(reshape2)

# Load data
gac.data <- read.csv("data/morphology.data_stickleback.csv", header = TRUE, stringsAsFactor = FALSE) # raw trait data

###################################################################################
## 1) Data transformation, size correction of traits, some summary stats of data ##
###################################################################################

# Log-transform traits (a few traits need log(trait + 1) transformation because of zeros in the data)
gac.all.data <- cbind.data.frame(ID = gac.data$ID,
                                 population = gac.data$population,
                                 type = gac.data$type,
                                 sex = gac.data$sex,
                                 size = log(gac.data$SL),
                                 pelvic.girdle.length = log(gac.data$pelvic.girdle.length + 1),
                                 pelvic.spine.length = log(gac.data$pelvic.spine.length),
                                 first.dorsal.spine.length = log(gac.data$first.dorsal.spine.length + 1),
                                 second.dorsal.spine.length = log(gac.data$second.dorsal.spine.length + 1),
                                 plate.number = log(gac.data$plate.number + 1),
                                 GR2.length = log(gac.data$GR2.length),
                                 GR3.length = log(gac.data$GR3.length),
                                 GR4.length = log(gac.data$GR4.length),
                                 gape.width = log(gac.data$gape.width),
                                 GR.count = log(gac.data$GR.count),
                                 premaxilla.length = log(gac.data$premaxilla.length),
                                 stringsAsFactors = FALSE)

# Size (SL, 'standard length') of all individuals per population (SupplMat Figure)
p <- ggplot(gac.data, aes(x = reorder(population, SL, FUN = median), y = SL, fill = type)) +
     xlab("Population") +
     ylab("Standard length (mm)") +
     geom_boxplot(outlier.shape = NA) +
     theme_bw(base_size = 12) +
     theme(axis.text.x = element_text(angle = 45, hjust = 1))
p

# Size correction (following Berner 2011) using standard length (SL) for size
sizeCorr = function(input_data, input_size, input_trait, input_grouping){
    xnew = input_data
    xnew[[input_size]] = mean(input_data[[input_size]], na.rm = T)
    z = lm(reformulate(c(input_size, input_grouping), input_trait), data = input_data, na.action = na.exclude)
    res = residuals(z)
    yhat = predict(z,xnew)
    size.corr.values = res + yhat
    input_data[[paste0(input_trait, ".sizeCorr")]] <- size.corr.values
    input_data
}
for(i in 6:dim(gac.all.data)[2]){
    gac.all.data <- sizeCorr(gac.all.data, "size", names(gac.all.data)[i], "population")
}
head(gac.all.data)

# Average the three gill raker (GR) length measurements and the two dorsal spine measurements
gac.all.data$GR.length.sizeCorr <- round(rowMeans(gac.all.data[,c("GR2.length.sizeCorr", "GR3.length.sizeCorr", "GR4.length.sizeCorr")], na.rm = TRUE),3)
gac.all.data$dorsal.spine.length.sizeCorr <- round(rowMeans(gac.all.data[,c("first.dorsal.spine.length.sizeCorr", "second.dorsal.spine.length.sizeCorr")], na.rm = TRUE),3)
gac.all.data <- gac.all.data[,-c(19,20,22:24)]

# Overview of sample sizes per trait and population: overall (males/females)
sample.size <- NULL

for(i in 17:length(gac.all.data[1,])){
    
    temp <- gac.all.data[!is.na(gac.all.data[,i]),]
    temp.f <- subset(temp, temp$sex == "f")
    temp.m <- subset(temp, temp$sex == "m")
    
    tot.per.pop.f <- table(temp.f$population)
    tot.per.pop.m <- table(temp.m$population)
    tot.per.pop <- table(temp$population)
    
    tot.per.pop.sex <- paste0(tot.per.pop, " (", tot.per.pop.f, "/", tot.per.pop.m, ")")
    out <- cbind.data.frame(names(tot.per.pop), tot.per.pop.sex)
    names(out)[1] <- "Population"
    names(out)[2] <- names(gac.all.data)[i]
    tot <- dim(temp)[1]
    total.ind <- data.frame(cbind("Total", tot))
    names(total.ind) <- names(out)
    out <- rbind(out, total.ind)
    
    if(i == 19){sample.size <- out}
    if(i != 19){sample.size <- cbind(sample.size, out[,2])}
    names(sample.size)[i-17] <- names(gac.all.data)[i]
}
sample.size


#####################################################################################
## 2) PCA and single-trait analyses of all antipredator traits and foraging traits ##
#####################################################################################

###################################################################################
# 2.1) Running the PCAs, PCA and single-trait summary and significance statistics #
###################################################################################

# Calculate population trait means
popMeans <- cbind.data.frame(
    dorsal.spine.length = tapply(gac.all.data$dorsal.spine.length.sizeCorr, gac.all.data$pop, mean, na.rm = T),
    pelvic.spine.length = tapply(gac.all.data$pelvic.spine.length.sizeCorr, gac.all.data$pop, mean, na.rm = T),
    pelvic.girdle.length = tapply(gac.all.data$pelvic.girdle.length.sizeCorr, gac.all.data$pop, mean, na.rm = T),
    plate.number = tapply(gac.all.data$plate.number.sizeCorr, gac.all.data$pop, mean, na.rm = T),
    gape.width = tapply(gac.all.data$gape.width.sizeCorr, gac.all.data$pop, mean, na.rm = T),
    GR.count = tapply(gac.all.data$GR.count.sizeCorr, gac.all.data$pop, mean, na.rm = T),
    GR.length = tapply(gac.all.data$GR.length.sizeCorr, gac.all.data$pop, mean, na.rm = T),
    premaxilla.length = tapply(gac.all.data$premaxilla.length.sizeCorr, gac.all.data$pop, mean, na.rm = T)
)
popMeans <- cbind(rownames(popMeans), popMeans)
names(popMeans)[1] <- "population"
popMeans

# PCA based on the correlation matrix of population means, calculated separately for antipredator and foraging traits
pca.antipredator <- prcomp(popMeans[,c("dorsal.spine.length", "pelvic.spine.length", "pelvic.girdle.length", "plate.number")], scale. = TRUE)
pca.foraging <- prcomp(popMeans[,c("gape.width", "GR.count", "GR.length", "premaxilla.length")], scale. = TRUE)

# Percent variance explained by PC1 and PC2
summary(pca.antipredator)
summary(pca.foraging)

# Trait loadings on PC1 and PC2
pca.antipredator$rotation
fviz_contrib(pca.antipredator, choice = "var", axes = 1, top = 4, title = "Contribution of traits on Antipredator PC1")
pca.foraging$rotation
fviz_contrib(pca.foraging, choice = "var", axes = 1, top = 4, title = "Contribution of traits on Foraging PC1")

# Extract PC1 and PC2 population values
foraging.pc1.popMeans <- pca.foraging$x[,1]
foraging.pc2.popMeans <- pca.foraging$x[,2]
antipredator.pc1.popMeans <- pca.antipredator$x[,1]
antipredator.pc2.popMeans <- pca.antipredator$x[,2]

# Restore meaningful trait units for reporting
final.data <- cbind.data.frame(popMeans$population, exp(popMeans[,-1]), foraging.pc1.popMeans, foraging.pc2.popMeans, antipredator.pc1.popMeans, antipredator.pc2.popMeans)
final.data[,grep("pelvic.girdle.length", names(final.data))] <- final.data[,grep("pelvic.girdle.length", names(final.data))]-1
final.data[,grep("dorsal.spine.length", names(final.data))] <- final.data[,grep("dorsal.spine.length", names(final.data))]-1
final.data[,grep("plate.number", names(final.data))] <- final.data[,grep("plate.number", names(final.data))]-1
lake.type <- c("sym", "allo", "sym", "allo", "allo", "allo", "allo", "allo", "sym", "sym", "allo", "sym", "sym", "sym", "sym", "sym", "allo", "allo")
final.data <- cbind(final.data$pop, lake.type, final.data[,c(2:dim(final.data)[2])])
names(final.data)[1] <- "population"

# Calculatate non-parametric P-values and 95% CI (takes 1-2 minutes to run on a desktop computer)
alp <- 0.05 # alpha-level
iter <- 10000 # number of resampling iterations

allo <- subset(final.data, final.data$lake.type == "allo")
symp <- subset(final.data, final.data$lake.type == "sym")

stats <- NULL
for(c in 3:length(final.data[1,])){
    allo.mean.obs <- round(mean(allo[,c]),3) # observed allopatric grand mean
    symp.mean.obs <- round(mean(symp[,c]),3) # observed sympatric grand mean
    a <- allo[,c]
    s <- symp[,c]
    
    # 95% CI
    a.rdist.mean <- rep(9, iter)
    s.rdist.mean <- rep(9, iter)
    for(i in 1:iter){
        a.rdist.mean[i] <- mean(sample(a, length(a), replace = T))
        s.rdist.mean[i] <- mean(sample(s, length(s), replace = T))
        }
    allo.95CI.low <- round(sort(a.rdist.mean)[alp/2*iter+1],2)
    allo.95CI.up <- round(sort(a.rdist.mean)[(1-(alp/2))*iter],2)
    symp.95CI.low <- round(sort(s.rdist.mean)[alp/2*iter+1],2)
    symp.95CI.up <- round(sort(s.rdist.mean)[(1-(alp/2))*iter],2)
    
    # P-value
    cnt <- 0
    for(i in 1:iter){
        x <- final.data
        x[,c] <- final.data[sample(1:length(final.data[,c])),c]
        allo.mean.rand <- mean(subset(x, x$lake.type == "allo")[,c])
        symp.mean.rand <- mean(subset(x, x$lake.type == "sym")[,c])
        if(abs(allo.mean.rand-symp.mean.rand) >= abs(allo.mean.obs-symp.mean.obs)){cnt <- cnt+1}
        Pvalue <- round(as.numeric(cnt/iter), digits = 3)
    }

    out <- rbind(names(final.data)[c], allo.mean.obs, allo.95CI.low, allo.95CI.up, symp.mean.obs, symp.95CI.low, symp.95CI.up, Pvalue)
    stats <- cbind(stats, out)
}

# Summary and significance statistics of all traits, PC1, and PC2
stats <- as.data.frame(stats)
names(stats) <- stats[1,]
stats <- stats[-1,]
stats <- cbind(cbind(c("NA", "NA", "NA", "NA", "NA", "NA", "NA"), c("NA", "NA", "NA", "NA", "NA", "NA", "NA")), stats)
names(stats)[1] <- names(final.data)[1]
names(stats)[2] <- names(final.data)[2]
final.data.rounded <- final.data %>% mutate_if(is.numeric, round, digits = 3)
overview <- rbind(final.data.rounded, stats)
overview

# It's good to see that parametric statistics produce very similar significance values for the difference between allopatric and sympatric lakes
# For example:
t.test(final.data$foraging.pc1.popMeans~final.data$lake.type)
t.test(final.data$antipredator.pc1.popMeans~final.data$lake.type)

###################################################################
# 2.2) Visualizing the major PC axes, incl. grand mean and 95% CI #
###################################################################

# Set parameters for calculating non-parametric 95% CI
alp <- 0.05
iter <- 10000

## Antipredator PC1:

# Reverse the arbitrary orientation of antipredator PC1 to be more intuitive
final.data$antipredator.pc1.popMeans <- final.data$antipredator.pc1.popMeans*(-1)

# Sympatric populations
d <- final.data[which(final.data$lake.type == "sym"), "antipredator.pc1.popMeans"]
symp.mean <- mean(d)
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.sym <- sort(rdist)[alp/2*iter+1]
upper.CI.sym <- sort(rdist)[(1-(alp/2))*iter]

# Allopatric populations
d <- final.data[which(final.data$lake.type == "allo"), "antipredator.pc1.popMeans"]
allo.mean <- mean(d)
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.allo <- sort(rdist)[alp/2*iter+1]
upper.CI.allo <- sort(rdist)[(1-(alp/2))*iter]

## Plot antipredator PC1 (raw version of Figure 2a)
p <- ggplot(final.data, aes(x = lake.type, y = antipredator.pc1.popMeans)) +
     labs(y = "Stickleback antipredator PC1", x = "Lake Type") +
     geom_jitter(position = position_jitter(0.05), shape = 1, cex = 3, stroke = 1.25) +
     geom_segment(aes(x = 1.25, y = low.CI.allo, xend = 1.25, yend = upper.CI.allo), colour = "grey", size = 0.75) +
     geom_segment(aes(x = 1.75, y = low.CI.sym, xend = 1.75, yend = upper.CI.sym), colour = "grey", size = 0.75) +
     geom_point(aes(x = 1.25, y = allo.mean), shape = 19, colour = "grey", size = 6) +
     geom_point(aes(x = 1.75, y = symp.mean), shape = 19, colour = "grey", size = 6) +
     theme_minimal()
p


## Foraging PC1:

# Reverse the arbitrary orientation of foraging PC1 to be more intuitive
final.data$foraging.pc1.popMeans <- final.data$foraging.pc1.popMeans*(-1)

# Sympatric populations
d <- final.data[which(final.data$lake.type == "sym"), "foraging.pc1.popMeans"]
symp.mean <- mean(d)
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.sym <- sort(rdist)[alp/2*iter+1]
upper.CI.sym <- sort(rdist)[(1-(alp/2))*iter]

# Allopatric populations
d <- final.data[which(final.data$lake.type == "allo"), "foraging.pc1.popMeans"]
allo.mean <- mean(d)
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.allo <- sort(rdist)[alp/2*iter+1]
upper.CI.allo <- sort(rdist)[(1-(alp/2))*iter]

# Plot foraging PC2 (raw version of Figure 2b)
p <- ggplot(final.data, aes(x = lake.type, y = foraging.pc1.popMeans)) +
     labs(y = "Stickleback foraging PC1", x = "Lake Type") +
     geom_jitter(position = position_jitter(0.05), shape = 1, cex = 3, stroke = 1.4) +
     geom_segment(aes(x = 1.25, y = low.CI.allo, xend = 1.25, yend = upper.CI.allo), colour = "grey", size = 0.75) +
     geom_segment(aes(x = 1.75, y = low.CI.sym, xend = 1.75, yend = upper.CI.sym), colour = "grey", size = 0.75) +
     geom_point(aes(x = 1.25, y = allo.mean), shape = 19, colour = "grey", size = 6) +
     geom_point(aes(x = 1.75, y = symp.mean), shape = 19, colour = "grey", size = 6) +
     theme_minimal()
p


####################################################################################
## 3) Alternative model-based test for trait differences between population types ##
## Used are log-transformed and size un-corrected (raw) trait values ###############
## Model syntax: lmer(trait ~ size + type + (1|population)) ########################
## Size = standard length (SL); "population" is added as random variable ###########
####################################################################################

gac.all.data.NoSizeCorr <- gac.all.data[,-grep(".sizeCorr", names(gac.all.data))]

# Average the three gill raker (GR) length measurements and the two dorsal spine measurements
gac.all.data.NoSizeCorr$GR.mean <- round(rowMeans(gac.all.data.NoSizeCorr[,c("GR2.length", "GR3.length", "GR4.length")], na.rm = TRUE),2)
gac.all.data.NoSizeCorr$dorsal.spine.length <- round(rowMeans(gac.all.data.NoSizeCorr[,c("first.dorsal.spine.length", "second.dorsal.spine.length")], na.rm = TRUE),2)
gac.all.data.NoSizeCorr <- gac.all.data.NoSizeCorr[,-c(8,9,11:13)]

## Run a linear-mixed model for each of the 4 antipredator and 4 foraging traits and collect statistics

collect.stats <- NULL
collect.EMM.stats <- NULL

# Antipredator trait 1/4: Pelvic girdle length
model <- lmer(pelvic.girdle.length ~ size + type + (1|population), data = gac.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("pelvic.girdle.length", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("pelvic.girdle.length", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Antipredator trait 2/4: Pelvic spine length
model <- lmer(pelvic.spine.length ~ size + type + (1|population), data = gac.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("pelvic.spine.length", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("pelvic.spine.length", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Antipredator trait 3/4: Plate number
model <- lmer(plate.number ~ size + type + (1|population), data = gac.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("plate.number", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("plate.number", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Antipredator trait 4/4: Dorsal spine length
model <- lmer(dorsal.spine.length ~ size + type + (1|population), data = gac.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("dorsal.spine.length", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("dorsal.spine.length", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Foraging trait 1/4: Gape width
model <- lmer(gape.width ~ size + type + (1|population), data = gac.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("gape.width", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("gape.width", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Foraging trait 2/4: Gill raker count
model <- lmer(GR.count ~ size + type + (1|population), data = gac.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("GR.count", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("GR.count", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Foraging trait 3/4: Gill raker length
model <- lmer(GR.mean ~ size + type + (1|population), data = gac.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("GR.length", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("GR.length", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Foraging trait 4/4: Premaxilla length
model <- lmer(premaxilla.length ~ size + type + (1|population), data = gac.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("premaxilla.length", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("premaxilla.length", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

collect.stats
collect.EMM.stats


#######################################################################################################
## 4) Test for trait correlations #####################################################################
## Calculate pairwise Pearson's correlations within each population using size-corrected trait data, ##
## then average these correlations across all populations #############################################
#######################################################################################################

# Subset to size-corrected data only
gac.all.data.SizeCorr <- cbind(gac.all.data[,c(1:5)], gac.all.data[,grep("sizeCorr", names(gac.all.data))])

# Reorder traits by type
gac.all.data.SizeCorr <- gac.all.data.SizeCorr[,c(1:8,13,9:12)]

# Calculate all pairwise trait correlations within each population, then average
populations <- unique(gac.all.data.SizeCorr$population)
X <- NULL
for(i in 1:length(populations)){
        dd <- subset(gac.all.data.SizeCorr, gac.all.data.SizeCorr$population == populations[i])
        out <- rcorr(as.matrix(dd[,c(6:dim(gac.all.data.SizeCorr)[2])]), type = c("pearson"))
        X <- cbind(X, out$r)
}
Y <- array(X, dim = c(dim(out[[1]]), length(populations)))
mean.corrMatrix <- data.frame(apply(Y, c(1,2), mean, na.rm = TRUE))
names(mean.corrMatrix) <- names(data.frame(out$r))
row.names(mean.corrMatrix) <- row.names(out$r)
round(mean.corrMatrix,3)
mean.corrMatrix_R <- mean.corrMatrix

# Prepare data for visualizing as heatmap (only get half of the correlation matrix as the other half is redundant)
get_lower_tri <- function(cormat){
    cormat[upper.tri(cormat)] <- NA
    return(cormat)}

lower_tri <- get_lower_tri(as.matrix(mean.corrMatrix_R))
melted_cormat <- melt(lower_tri, na.rm = TRUE)
names(melted_cormat)[1] <- "value1"
names(melted_cormat)[2] <- "value2"
melted_cormat[,3] <- round(melted_cormat[,3],2)

# Visualize heatmap (SupplMat Figure)
ggheatmap <- ggplot(melted_cormat, aes(value1, value2, fill = value)) +
  geom_tile(color = "white") +
  scale_fill_gradient2(low = "blue", high = "red", mid = "white",
  midpoint = 0, limit = c(-1, 1), space = "Lab", name = "Pearson\nCorrelation") +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 30, vjust = 1, size = 12, hjust = 1)) +
  coord_fixed()

p <- ggheatmap +
     geom_text(aes(value1, value2, label = value), color = "black", size = 3) +
     theme(axis.title.x = element_blank(),
        axis.title.y = element_blank(),
        panel.grid.major = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        axis.ticks = element_blank(),
        legend.justification = c(1, 0),
        legend.position = c(0.4, 0.7),
        legend.direction = "horizontal") +
     guides(fill = guide_colorbar(barwidth = 7, barheight = 1, title.position = "top", title.hjust = 0.5))
p


########################################################################################################################################
########################################################################################################################################


#####################################################
#####################################################
### SCULPIN MORPHOLOGICAL TRAIT ANALYSES ############
### Code used for Roesti et al. (Ecology Letters) ###
#####################################################
#####################################################

rm(list = ls(all = TRUE))

# Set path to the downloaded dryad folder containing all codes and data for Roesti et al.
setwd("XXXX")

# Load packages
library(MASS)
library(FactoMineR)
library(factoextra)
library(corrplot)
library(car)
library(ppcor)
library(emmeans)
library(ggplot2)
library(lme4)
library(dplyr)
library(Hmisc)
library(reshape2)

# Load data
cas.data <- read.csv("data/morphology.data_sculpin.csv", header = TRUE, stringsAsFactor = FALSE) # raw trait data

##############################################################################
## 1) Data transformation, size correction of traits, summary stats of data ##
##############################################################################

# Log-transform traits
cas.all.data <- cbind.data.frame(ID = cas.data$ID,
                                 population = cas.data$population,
                                 type = cas.data$type,
                                 sex = cas.data$sex,
                                 size = log(cas.data$SL),
                                 body.width = log(cas.data$body.width),
                                 dorsal.spine.length = log(cas.data$dorsal.spine.length),
                                 pelvic.spine.length = log(cas.data$pelvic.spine.length),
                                 preop.spine.length = log(cas.data$preop.spine.length),
                                 dorsal.fin.area = log(cas.data$dorsal.fin.area),
                                 anal.fin.area = log(cas.data$anal.fin.area),
                                 caudal.fin.area = log(cas.data$caudal.fin.area),
                                 peduncle.depth = log(cas.data$peduncle.depth),
                                 GR2.length = log(cas.data$GR2.length),
                                 GR3.length = log(cas.data$GR3.length),
                                 GR4.length = log(cas.data$GR4.length),
                                 gape.width = log(cas.data$gape.width),
                                 GR.count = log(cas.data$GR.count),
                                 premaxilla.length = log(cas.data$premaxilla.length),
                                 stringsAsFactors = FALSE)

# Size (SL, 'standard length') of all individuals per population (SupplMat Figure)
p <- ggplot(cas.data, aes(x = reorder(population, SL, FUN = median), y = SL, fill = type)) +
     xlab("Population") +
     ylab("Standard length (mm)") +
     geom_boxplot(outlier.shape = NA) +
     theme_bw(base_size = 12) +
     theme(axis.text.x = element_text(angle = 45, hjust = 1))
p

# Size correction (following Berner 2011) using standard length (SL) for size
sizeCorr = function(input_data, input_size, input_trait, input_grouping){
    xnew = input_data
    xnew[[input_size]] = mean(input_data[[input_size]], na.rm = T)
    z = lm(reformulate(c(input_size, input_grouping), input_trait), data = input_data, na.action = na.exclude)
    res = residuals(z)
    yhat = predict(z,xnew)
    size.corr.values = res + yhat
    input_data[[paste0(input_trait, ".sizeCorr")]] <- size.corr.values
    input_data
}
for(i in 6:dim(cas.all.data)[2]){
    cas.all.data <- sizeCorr(cas.all.data, "size", names(cas.all.data)[i], "population")
}
head(cas.all.data)

# Average the three gill raker (GR) length measurements and then remove the single measurements
cas.all.data$GR.mean.sizeCorr <- round(rowMeans(cas.all.data[, c("GR2.length.sizeCorr", "GR3.length.sizeCorr", "GR4.length.sizeCorr")], na.rm = TRUE),3)
cas.all.data <- cas.all.data[, -c(28:30)]

# Overview of sample sizes per trait and population: overall (males/females)
sample.size <- NULL

for(i in 20:length(cas.all.data[1, ])){
    
    temp <- cas.all.data[!is.na(cas.all.data[,i]), ]
    temp.f <- subset(temp, temp$sex == "f")
    temp.m <- subset(temp, temp$sex == "m")

    tot.per.pop.f <- table(temp.f$population)
    tot.per.pop.m <- table(temp.m$population)
    tot.per.pop <- table(temp$population)
    
    tot.per.pop.sex <- paste0(tot.per.pop, " (", tot.per.pop.f, "/", tot.per.pop.m, ")")
    out <- cbind.data.frame(names(tot.per.pop), tot.per.pop.sex)
    names(out)[1] <- "Population"
    names(out)[2] <- names(cas.all.data)[i]
    tot <- dim(temp)[1]
    total.ind <- data.frame(cbind("Total", tot))
    names(total.ind) <- names(out)
    out <- rbind(out, total.ind)
    
    if(i == 20){sample.size <- out}
    if(i != 20){sample.size <- cbind(sample.size, out[,2])}
    names(sample.size)[i-18] <- names(cas.all.data)[i]
}
sample.size


#####################################################################################
## 2) PCA and single-trait analysis of all antipredator traits and foraging traits ##
#####################################################################################

#############################################################################################
# 2.1) Running the PCAs, obtaining PCA and single-trait summary and significance statistics #
#############################################################################################

# Calculate population means of size-corrected trait values
popMeans <- cbind.data.frame(
  dorsal.spine.length = tapply(cas.all.data$dorsal.spine.length.sizeCorr, cas.all.data$population, mean, na.rm = T),
  pelvic.spine.length = tapply(cas.all.data$pelvic.spine.length.sizeCorr, cas.all.data$population, mean, na.rm = T),
  preoperc.spine.length = tapply(cas.all.data$preop.spine.length.sizeCorr, cas.all.data$population, mean, na.rm = T),
  caudal.fin.area = tapply(cas.all.data$caudal.fin.area.sizeCorr, cas.all.data$population, mean, na.rm = T),
  dorsal.fin.area = tapply(cas.all.data$dorsal.fin.area.sizeCorr, cas.all.data$population, mean, na.rm = T),
  anal.fin.area = tapply(cas.all.data$anal.fin.area.sizeCorr, cas.all.data$population, mean, na.rm = T),
  peduncle.depth = tapply(cas.all.data$peduncle.depth.sizeCorr, cas.all.data$population, mean, na.rm = T),
  body.width = tapply(cas.all.data$body.width.sizeCorr, cas.all.data$population, mean, na.rm = T),
  gape.width = tapply(cas.all.data$gape.width.sizeCorr, cas.all.data$population, mean, na.rm = T),
  GR.count = tapply(cas.all.data$GR.count.sizeCorr, cas.all.data$population, mean, na.rm = T),
  GR.length = tapply(cas.all.data$GR.mean.sizeCorr, cas.all.data$population, mean, na.rm = T),
  premaxilla.length = tapply(cas.all.data$premaxilla.length.sizeCorr, cas.all.data$population, mean, na.rm = T)
)
popMeans <- cbind(rownames(popMeans), popMeans)
names(popMeans)[1] <- "population"
popMeans

# PCA based on the correlation matrix of population means, calculated separately for antipredator and foraging traits
pca.antipredator <- prcomp(popMeans[, c("dorsal.spine.length", "pelvic.spine.length", "preoperc.spine.length", "caudal.fin.area", "dorsal.fin.area", "anal.fin.area", "peduncle.depth", "body.width")], scale. =  TRUE)
pca.foraging <- prcomp(popMeans[, c("gape.width", "GR.count", "GR.length", "premaxilla.length")], scale. =  TRUE)

# Percent variance explained by PC1 and PC2
summary(pca.antipredator)
summary(pca.foraging)

# Trait loadings on PC1 and PC2 (note that for Figure 2b, the arbitrary orientation of the foraging PC2 axis will be reversed, see below)
pca.antipredator$rotation
fviz_contrib(pca.antipredator, choice = "var", axes = 1, top = 8, title = "Contribution of traits on Antipredator PC1")
pca.foraging$rotation
fviz_contrib(pca.foraging, choice = "var", axes = 2, top = 4, title = "Contribution of traits on Foraging PC1")

# Extract PC1 and PC2 population values
antipredator.pc1.popMeans <- pca.antipredator$x[,1]
antipredator.pc2.popMeans <- pca.antipredator$x[,2]
foraging.pc1.popMeans <- pca.foraging$x[,1]
foraging.pc2.popMeans <- pca.foraging$x[,2]

# Restore meaningful trait units for reporting
final.data <- cbind.data.frame(popMeans$pop, exp(popMeans[,-1]), antipredator.pc1.popMeans, antipredator.pc2.popMeans, foraging.pc1.popMeans, foraging.pc2.popMeans)
lake.type <- c("sym", "allo", "allo", "sym", "allo", "allo", "allo", "sym", "sym", "sym", "allo", "sym", "sym", "sym", "sym", "allo", "allo")
final.data <- cbind(final.data$pop, lake.type, final.data[, c(2:dim(final.data)[2])])
names(final.data)[1] <- "population"

# Calculatate non-parametric P-values and 95% CI (takes ~2 minutes to run on a desktop computer)
alp <- 0.05 # alpha-level
iter <- 10000 # number of resampling iterations

allo <- subset(final.data, final.data$lake.type == "allo")
symp <- subset(final.data, final.data$lake.type == "sym")

stats <- NULL
for(c in 3:length(final.data[1,])){
    allo.mean.obs <- round(mean(allo[,c]),3) # observed allopatric grand mean
    symp.mean.obs <- round(mean(symp[,c]),3) # observed sympatric grand mean
    a <- allo[,c]
    s <- symp[,c]
    
    # 95% CI
    a.rdist.mean <- rep(9, iter)
    s.rdist.mean <- rep(9, iter)
    for(i in 1:iter){
        a.rdist.mean[i] <- mean(sample(a, length(a), replace = T))
        s.rdist.mean[i] <- mean(sample(s, length(s), replace = T))
        }
    allo.95CI.low <- round(sort(a.rdist.mean)[alp/2*iter+1],2)
    allo.95CI.up <- round(sort(a.rdist.mean)[(1-(alp/2))*iter],2)
    symp.95CI.low <- round(sort(s.rdist.mean)[alp/2*iter+1],2)
    symp.95CI.up <- round(sort(s.rdist.mean)[(1-(alp/2))*iter],2)
    
    # P-value
    cnt <- 0
    for(i in 1:iter){
        x <- final.data
        x[,c] <- final.data[sample(1:length(final.data[, c])),c]
        allo.mean.rand <- mean(subset(x, x$lake.type == "allo")[,c])
        symp.mean.rand <- mean(subset(x, x$lake.type == "sym")[,c])
        if(abs(allo.mean.rand-symp.mean.rand) >= abs(allo.mean.obs-symp.mean.obs)){cnt <- cnt+1}
        Pvalue <- round(as.numeric(cnt/iter), digits = 3)
    }

    out <- rbind(names(final.data)[c], allo.mean.obs, allo.95CI.low, allo.95CI.up, symp.mean.obs, symp.95CI.low, symp.95CI.up, Pvalue)
    stats <- cbind(stats, out)
}

# Summary and significance statistics of all traits, PC1, and PC2
stats <- as.data.frame(stats)
names(stats) <- stats[1,]
stats <- stats[-1,]
stats <- cbind(cbind(c("NA", "NA", "NA", "NA", "NA", "NA", "NA"), c("NA", "NA", "NA", "NA", "NA", "NA", "NA")), stats)
names(stats)[1] <- names(final.data)[1]
names(stats)[2] <- names(final.data)[2]
final.data.rounded <- final.data %>% mutate_if(is.numeric, round, digits = 3)
overview <- rbind(final.data.rounded, stats)
overview

# It's good to see that parametric statistics produce very similar significance values for the difference between allopatric and sympatric lakes
# For example:
t.test(final.data$foraging.pc2.popMeans~final.data$lake.type)
t.test(final.data$antipredator.pc1.popMeans~final.data$lake.type)

########################################################################################
# 2.2) Visualizing the major PC axes, incl. grand means of lake types and their 95% CI #
########################################################################################

# Set parameters for calculating non-parametric 95% CI
alp <- 0.05
iter <- 10000

## Antipredator PC1:

# Sympatric populations
d <- final.data[which(final.data$lake.type == "sym"), "antipredator.pc1.popMeans"]
symp.mean <- mean(d)
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.sym <- sort(rdist)[alp/2*iter+1]
upper.CI.sym <- sort(rdist)[(1-(alp/2))*iter]

# Allopatric populations
d <- final.data[which(final.data$lake.type == "allo"), "antipredator.pc1.popMeans"]
allo.mean <- mean(d)
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.allo <- sort(rdist)[alp/2*iter+1]
upper.CI.allo <- sort(rdist)[(1-(alp/2))*iter]

# Plot antipredator PC1 (raw version of Figure 2a)
p <- ggplot(final.data, aes(x = lake.type, y = antipredator.pc1.popMeans)) +
     labs(y = "Sculpin antipredator PC1", x = "Lake Type") +
     geom_jitter(position = position_jitter(0.05), shape = 1, cex = 3, stroke = 1.25) +
     geom_segment(aes(x = 1.25, y = low.CI.allo, xend = 1.25, yend = upper.CI.allo), colour = "grey", size = 0.75) +
     geom_segment(aes(x = 1.75, y = low.CI.sym, xend = 1.75, yend = upper.CI.sym), colour = "grey", size = 0.75) +
     geom_point(aes(x = 1.25, y = allo.mean), shape = 19, colour = "grey", size = 6) +
     geom_point(aes(x = 1.75, y = symp.mean), shape = 19, colour = "grey", size = 6) +
     theme_minimal()
p

## Foraging PC2 (foraging PC1 is non-significant):

# Reverse the arbitrary orientation of foraging PC2 to be more intuitive
final.data$foraging.pc2.popMeans <- final.data$foraging.pc2.popMeans*(-1)

# Sympatric populations
d <- final.data[which(final.data$lake.type == "sym"), "foraging.pc2.popMeans"]
symp.mean <- mean(d)
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.sym <- sort(rdist)[alp/2*iter+1]
upper.CI.sym <- sort(rdist)[(1-(alp/2))*iter]

# Allopatric populations
d <- final.data[which(final.data$lake.type == "allo"), "foraging.pc2.popMeans"]
allo.mean <- mean(d)
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.allo <- sort(rdist)[alp/2*iter+1]
upper.CI.allo <- sort(rdist)[(1-(alp/2))*iter]

# Plot foraging PC2 (raw version of Figure 2b)
p <- ggplot(final.data, aes(x = lake.type, y = foraging.pc2.popMeans)) +
     labs(y = "Sculpin foraging PC2", x = "Lake Type") +
     geom_jitter(position = position_jitter(0.05), shape = 1, cex = 3, stroke = 1.4) +
     geom_segment(aes(x = 1.25, y = low.CI.allo, xend = 1.25, yend = upper.CI.allo), colour = "grey", size = 0.75) +
     geom_segment(aes(x = 1.75, y = low.CI.sym, xend = 1.75, yend = upper.CI.sym), colour = "grey", size = 0.75) +
     geom_point(aes(x = 1.25, y = allo.mean), shape = 19, colour = "grey", size = 6) +
     geom_point(aes(x = 1.75, y = symp.mean), shape = 19, colour = "grey", size = 6) +
     theme_minimal()
p


####################################################################################
## 3) Alternative model-based test for trait differences between population types ##
## Used are log-transformed and size un-corrected (raw) trait values ###############
## Model syntax: lmer(trait ~ size + type + (1|population)) ########################
## Size = standard length (SL); "population" is added as random variable ###########
####################################################################################

cas.all.data.NoSizeCorr <- cas.all.data[,-grep(".sizeCorr", names(cas.all.data))]

# Average the three gill raker (GR) length measurements
cas.all.data.NoSizeCorr$GR.mean <- round(rowMeans(cas.all.data.NoSizeCorr[, c("GR2.length", "GR3.length", "GR4.length")], na.rm = TRUE),4)
cas.all.data.NoSizeCorr <- cas.all.data.NoSizeCorr[,-c(14:16)]

## Run a linear-mixed model for each of the 8 antipredator and 4 foraging traits and collect statistics

collect.stats <- NULL
collect.EMM.stats <- NULL

# Antipredator trait 1/8: Body width
model <- lmer(body.width ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("body.width", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("body.width", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Antipredator trait 2/8: Dorsal spine length
model <- lmer(dorsal.spine.length ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("dorsal.spine.length", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("dorsal.spine.length", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Antipredator trait 3/8: Pelvic spine length
model <- lmer(pelvic.spine.length ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("pelvic.spine.length", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("pelvic.spine.length", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Antipredator trait 4/8: Preopercular spine length
model <- lmer(preop.spine.length ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("preop.spine.length", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("preop.spine.length", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Antipredator trait 5/8: Dorsal fin area
model <- lmer(dorsal.fin.area ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("dorsal.fin.area", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("dorsal.fin.area", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Antipredator trait 6/8: Caudal fin area
model <- lmer(caudal.fin.area ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("caudal.fin.area", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("caudal.fin.area", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Antipredator trait 7/8: Anal fin area
model <- lmer(anal.fin.area ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("anal.fin.area", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("anal.fin.area", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Antipredator trait 8/8: Peduncle depth
model <- lmer(peduncle.depth ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("peduncle.depth", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("peduncle.depth", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Foraging trait 1/4: Gape width
model <- lmer(gape.width ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("gape.width", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("gape.width", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Foraging trait 2/4: Gill raker count
model <- lmer(GR.count ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("GR.count", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("GR.count", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Foraging trait 3/4: Gill raker length
model <- lmer(GR.mean ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("GR.length", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("GR.length", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

# Foraging trait 4/4: Premaxilla length
model <- lmer(premaxilla.length ~ size + type + (1|population), data = cas.all.data.NoSizeCorr)
p.val <- round(Anova(model, type = "III")$Pr[3],3)
estimate <- round(coef(summary(model))[3,1],3)
s.e. <- round(coef(summary(model))[3,2],3)
t.value <- round(coef(summary(model))[3,3],3)
out <- cbind.data.frame("premaxilla.length", p.val, estimate, s.e., t.value)
names(out) <- c("trait", "P-value", "Estimate", "S.E.", "t-value")
collect.stats <- rbind(collect.stats, out)

emmans.allo <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,2]),2)
emmans.allo.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[1,6]),2))
emmans.sym <- round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,2]),2)
emmans.sym.CI <- paste0(round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,5]),2), "–", round(exp(data.frame(emmeans(model, "type", weights = "equal"))[2,6]),2))
out <- cbind.data.frame("premaxilla.length", "antipredator", emmans.allo, emmans.allo.CI, emmans.sym, emmans.sym.CI)
names(out) <- c("Trait", "Trait type", "Allopatric EMM", "Allopatric EMM 95% CI", "Sympatric EMM", "Sympatric EMM 95% CI")
collect.EMM.stats <- rbind(collect.EMM.stats, out)

collect.stats
collect.EMM.stats


#######################################################################################################
## 4) Test for trait correlations #####################################################################
## Calculate pairwise Pearson's correlations within each population using size-corrected trait data, ##
## then average these correlations across all populations ######### ###################################
#######################################################################################################

# Subset to size-corrected data only
cas.all.data.SizeCorr <- cbind(cas.all.data[, c(1:5)], cas.all.data[,grep("sizeCorr", names(cas.all.data))])

# Calculate all pairwise trait correlations within each population, then average across populations
populations <- unique(cas.all.data.SizeCorr$population)
X <- NULL
for(i in 1:length(populations)){
        dd <- subset(cas.all.data.SizeCorr, cas.all.data.SizeCorr$population == populations[i])
        out <- rcorr(as.matrix(dd[, c(7:dim(cas.all.data.SizeCorr)[2])]), type = c("pearson"))
        X <- cbind(X, out$r)
}
Y <- array(X, dim = c(dim(out[[1]]), length(populations)))
mean.corrMatrix <- data.frame(apply(Y, c(1,2), mean, na.rm = TRUE))
names(mean.corrMatrix) <- names(data.frame(out$r))
row.names(mean.corrMatrix) <- row.names(out$r)
round(mean.corrMatrix,3)
mean.corrMatrix_R <- mean.corrMatrix

# Prepare data for visualization as heatmap
get_lower_tri <- function(cormat){
    cormat[upper.tri(cormat)] <- NA
    return(cormat)}

lower_tri <- get_lower_tri(as.matrix(mean.corrMatrix_R))
melted_cormat <- melt(lower_tri, na.rm = TRUE)
names(melted_cormat)[1] <- "value1"
names(melted_cormat)[2] <- "value2"
melted_cormat[,3] <- round(melted_cormat[,3],2)

# Visualize heatmap (SupplMat Figure)
ggheatmap <- ggplot(melted_cormat, aes(value1, value2, fill = value)) +
  geom_tile(color = "white") +
  scale_fill_gradient2(low = "blue", high = "red", mid = "white",
  midpoint = 0, limit = c(-1, 1), space = "Lab", name = "Pearson\nCorrelation") +
  theme_minimal() +
  theme(axis.text.x = element_text(angle = 30, vjust = 1, size = 12, hjust = 1)) +
  coord_fixed()

p <- ggheatmap +
     geom_text(aes(value1, value2, label = value), color = "black", size = 3) +
     theme(axis.title.x = element_blank(),
        axis.title.y = element_blank(),
        panel.grid.major = element_blank(),
        panel.border = element_blank(),
        panel.background = element_blank(),
        axis.ticks = element_blank(),
        legend.justification = c(1, 0),
        legend.position = c(0.4, 0.7),
        legend.direction = "horizontal") +
     guides(fill = guide_colorbar(barwidth = 7, barheight = 1, title.position = "top", title.hjust = 0.5))
p


########################################################################################################################################
########################################################################################################################################


#####################################################
#####################################################
### VISUALIZE TRAIT-SPECIFIC SHIFTS (FIG. 3) ########
### Code used for Roesti et al. (Ecology Letters) ###
#####################################################
#####################################################

rm(list = ls(all = TRUE))

# Set path to the downloaded dryad folder containing all information (code and data) for Roesti et al.
setwd("XXXX")

# Load package
library(ggplot2)

# Load data
gac.data <- read.csv("data/morphology.population.means_stickleback.csv", header = TRUE, stringsAsFactor = FALSE) # stickleback population trait means as obtained from running the code above
cas.data <- read.csv("data/morphology.population.means_sculpin.csv", header = TRUE, stringsAsFactor = FALSE) # sculpin population trait means as obtained from running the code above


## Prepare stickleback data for plotting

allo <- subset(gac.data, gac.data$lake.type == "allo")
sym <- subset(gac.data, gac.data$lake.type == "sym")

out.gac <- NULL
for(i in 3:10){
    if(mean(allo[, i])>mean(sym[,i])){
        mean.allo <- round(mean(allo[,i]),3)
        mean.sym <- round(mean(sym[,i]),3)
        percentage <- -round((mean(allo[,i])/mean(sym[,i])*100)-100,1)
        info <- cbind("stickleback", names(allo)[i], percentage, mean.sym, mean.allo)
        out.gac <- rbind(out.gac, info)}
    if(mean(allo[, i])<mean(sym[, i])){
        if(length(grep("plate", names(sym)[i])) == 1){
            sym.excl.North <- subset(sym, sym$population != "North")
            mean.allo <- round(mean(allo[,i]),3)
            mean.sym <- round(mean(sym.excl.North[,i]),3)
            percentage <- round(((mean.sym/mean.allo)*100)-100,1)
            info <- cbind("stickleback", names(allo)[i], percentage, mean.sym, mean.allo)
            out.gac <- rbind(out.gac, info)}
        if(length(grep("plate", names(sym)[i]))!= 1){
            mean.allo <- round(mean(allo[,i]),3)
            mean.sym <- round(mean(sym[,i]),3)
            percentage <- round(((mean.sym/mean.allo)*100)-100,1)
            info <- cbind("stickleback", names(allo)[i], percentage, mean.sym, mean.allo)
            out.gac <- rbind(out.gac, info)}
            }
}
out.gac <- data.frame(out.gac)
names(out.gac) <- c("species", "trait", "percent.change.in.sympatry", "mean.sym", "mean.allo")


## Prepare sculpin data for plotting

allo <- subset(cas.data, cas.data$lake.type == "allo")
sym <- subset(cas.data, cas.data$lake.type == "sym")

out.cas <- NULL
for(i in 3:14){
    if(mean(allo[, i])>mean(sym[,i])){
        mean.allo <- round(mean(allo[, i]),3)
        mean.sym <- round(mean(sym[, i]),3)
        percentage <- -round((mean(allo[, i])/mean(sym[, i])*100)-100,1)
        info <- cbind("sculpin", names(allo)[i], percentage, mean.sym, mean.allo)
        out.cas <- rbind(out.cas, info)}
    
    if(mean(allo[, i])<mean(sym[, i])){
            mean.allo <- round(mean(allo[,i]),3)
            mean.sym <- round(mean(sym[,i]),3)
            percentage <- round(((mean.sym/mean.allo)*100)-100,1)
            info <- cbind("sculpin", names(allo)[i], percentage, mean.sym, mean.allo)
            out.cas <- rbind(out.cas, info)}
}
out.cas <- data.frame(out.cas)
names(out.cas) <- c("species", "trait", "percent.change.in.sympatry", "mean.sym", "mean.allo")
out <- rbind(out.gac, out.cas)


## Order traits and then plot trait shifts (raw version of Fig. 3)

out[,3] <- as.numeric(out[,3])
positions <- c("premaxilla.length", "gape.width", "GR.length", "GR.count", "body.width", "dorsal.fin.area", "caudal.fin.area", "anal.fin.area", "peduncle.depth", "preoperc.spine.length", "plate.number", "pelvic.girdle.length", "pelvic.spine.length", "dorsal.spine.length")

ggplot(data = out, aes(y = percent.change.in.sympatry, x = trait, fill = species)) +
    scale_y_continuous("% change in sympatry", limits = c(-15, 70)) +
    scale_x_discrete(limits = positions) +
    geom_bar(stat = "identity", width = 0.5) +
    scale_fill_brewer(palette = "Paired") +
    theme_minimal() +
    theme(legend.position = "none") +
    coord_flip()
