#############################################################
#############################################################
### RE-ANALYSIS OF THE EXPERIMENT BY BOLNICK ET AL.(2010) ###
### Code used for Roesti et al. (Ecology Letters) ###########
#############################################################
#############################################################

rm(list = ls(all = TRUE))

# Set path to the downloaded dryad folder containing all codes and data for Roesti et al.
setwd("XXXX")

# Load packages
library(car)
library(MASS)
library(data.table)
library(ggplot2)
library(dplyr)

# Load data
match.with.wild <- read.csv("data/diet.categories_bolnick.reanalysis.csv", header = TRUE, stringsAsFactor = FALSE)
data <- read.csv("data/experimental.data_bolnick.reanalysis.csv", header = TRUE, stringsAsFactor = FALSE)
reference <- read.csv("data/trout&sculpin.diet_bolnick.reanalysis.csv", header = TRUE, stringsAsFactor = FALSE)


##################################
## 1) Prepare data for analysis ##
##################################

## 1.1) Adjusting the stickleback data

dietitems <- data[,10:49] # columns with the prey data
names(dietitems)
dietitems.sort <- dietitems[, order(colnames(dietitems))] # order by alphabet

# Adjust names of stickleback prey categories
dietitems.sort <- dietitems.sort[,-grep("Egg.capsule", names(dietitems.sort))] # remove "Egg.capsule" because this category is ambiguous (note that there is an own category for "stickleback eggs")
names(dietitems.sort)[grep("hatchling.stickleback", names(dietitems.sort))] <- "Stickleback.larvae" # adjust name

# Collapse "Ephemeroptera.pupae" and "Ephemeroptera.larvae" into "Ephemeroptera"
dietitems.sort[, "Ephemeroptera.larvae"] <- dietitems.sort[, grep("Ephemeroptera.pupae", names(dietitems.sort))] + dietitems.sort[, grep("Ephemeroptera.larvae", names(dietitems.sort))]
names(dietitems.sort)[grep("Ephemeroptera.larvae", names(dietitems.sort))] <- "Ephemeroptera"
dietitems.sort <- dietitems.sort[,-grep("Ephemeroptera.pupae", names(dietitems.sort))]

# Collapse "Trichoptera.pupae" and "Trichoptera.adult" into "Trichoptera"
dietitems.sort[, "Trichoptera.pupae"] <- dietitems.sort[, grep("Trichoptera.pupae", names(dietitems.sort))] + dietitems.sort[, grep("Trichoptera.larvae", names(dietitems.sort))] + dietitems.sort[, grep("Trichoptera.adult", names(dietitems.sort))]
names(dietitems.sort)[grep("Trichoptera.pupae", names(dietitems.sort))] <- "Trichoptera"
dietitems.sort <- dietitems.sort[,-grep("Trichoptera.larvae", names(dietitems.sort))]
dietitems.sort <- dietitems.sort[,-grep("Trichoptera.adult", names(dietitems.sort))]

# Remove very rare prey categories found in only two or less stickleback individuals unless it is a prey category that was also found in sculpin and/or trout
dietitems.sort <- dietitems.sort[,-grep("Brown.capsule", names(dietitems.sort))]
dietitems.sort <- dietitems.sort[,-grep("Blatteria", names(dietitems.sort))]
dietitems.sort <- dietitems.sort[,-grep("Cestode", names(dietitems.sort))]
dietitems.sort <- dietitems.sort[,-grep("Diptera.larvae", names(dietitems.sort))]
dietitems.sort <- dietitems.sort[,-grep("Earwig", names(dietitems.sort))]
dietitems.sort <- dietitems.sort[,-grep("Empididae", names(dietitems.sort))]
dietitems.sort <- dietitems.sort[,-grep("Hymenoptera", names(dietitems.sort))]
dietitems.sort <- dietitems.sort[,-grep("Staphylinidae", names(dietitems.sort))]
dietitems.sort <- dietitems.sort[,-grep("Tanyderidae", names(dietitems.sort))]

# Which and how many prey categories are there now for stickleback?
row.names(data.frame(colSums(dietitems.sort != 0)))


## 1.2) Adjusting the sculpin/trout data

names(reference)[grep("Anisoptera", names(reference))] <- "Anisoptera.larvae" # adjust name so that it matches with the respective stickleback prey category
reference <- reference[,-grep("Unknown.adult.insect", names(reference))] # remove the "Unknown.adult.insect" category because it's ambiguous
reference <- reference[,-grep("Stratiomyidae.lar.", names(reference))] # remove the "Stratiomyidae.lar." category because it was found only once in sculpin/trout and never in stickleback
reference <- reference[,-grep("Spider", names(reference))] # remove the "spider" category because it was found only once in sculpin/trout and never in stickleback

# Which and how many prey categories are there now for sculpin/trout?
row.names(data.frame(colSums(reference[, c(5:length(reference[1,]))] != 0)))


## 1.3) Align stickleback and trout/sculpin data and calculate individual diet proportions

# Remove stickleback individuals with empty stomachs (N = 55)
remove.emptyIndiv <- which(rowSums(dietitems.sort) == 0)
dietitems.sort <- dietitems.sort[-remove.emptyIndiv,]
data <- data[-remove.emptyIndiv,]
dim(data)[1] == dim(dietitems.sort)[1] # check: "data" and "dietitems.sort" should have the same number of rows. Looking good.

# Calculate prey item proportions per stickleback individual
stickle.dietproportions <- dietitems.sort/rowSums(dietitems.sort)
unique(rowSums(stickle.dietproportions)) # check: should only be 1. Looking good.

# Check: are all trout/sculpin prey categories present in the stickleback dataset?
st.prey <- names(stickle.dietproportions)
length(st.prey) # number of stickleback prey categories
sc.tr.prey <- names(reference[, c(5:length(reference[1,]))])
length(sc.tr.prey) # number of trout/sculpin prey categories
length(intersect(sc.tr.prey, st.prey)) == length(sc.tr.prey) # looking good

# For prey categories found in only stickleback, add empty columns to the trout/sculpin dataset
unique.st.prey <- setdiff(st.prey, sc.tr.prey)
addMatrix <- data.frame(matrix(0, length(reference[,1]), length(unique.st.prey)))
names(addMatrix) <- unique.st.prey
reference <- cbind(reference, addMatrix)

# Sculpin diet
sculpin.prey <- subset(reference, reference$Species == "Sculpin")[, -c(1:4)] # subset to only sculpin
sculpin.prey <- sculpin.prey[, order(match(names(sculpin.prey), names(stickle.dietproportions)))]
unique(names(sculpin.prey) == names(stickle.dietproportions)) # check whether only "TRUE". If so, the order of the prey categories between stickles and sculpin match. Looking good.
sculpin.prey <- sculpin.prey[-which(rowSums(sculpin.prey) == 0),] # remove the two sculpin individuals with empty stomachs
sculpindiet.proportions <- sculpin.prey/rowSums(sculpin.prey)
sculpindiet <- colMeans(sculpindiet.proportions) # average sculpin diet

# Trout diet
trout.prey <- subset(reference, reference$Species == "Trout")[, -c(1:4)] # subset to only trout
trout.prey <- trout.prey[, order(match(names(trout.prey), names(stickle.dietproportions)))]
unique(names(trout.prey) == names(stickle.dietproportions)) # check whether only "TRUE". If so, the order of the prey categories between stickles and trout match. Looking good.
troutdiet.proportions <- trout.prey/rowSums(trout.prey)
troutdiet <- colMeans(troutdiet.proportions) # average trout diet

# Clean up before moving on
rm("dietitems", "remove.emptyIndiv", "sc.tr.prey", "sculpin.prey", "st.prey", "trout.prey", "addMatrix", "unique.st.prey")


#################################################################################################################
## 2) Calculate proportional similarity (PSi) between each stickleback and the average trout and sculpin diets ##
#################################################################################################################

PSi_trout <- c()
PSi_sculpin <- c()
for(i in 1:dim(stickle.dietproportions)[1]){
  stickle_i <- as.numeric(stickle.dietproportions[i,])
  PSi_trout <- c(PSi_trout, sum(pmin(stickle_i, troutdiet)))
  PSi_sculpin <- c(PSi_sculpin, sum(pmin(stickle_i, sculpindiet)))
}

data$PSi_trout <- PSi_trout
data$PSi_sculpin <- PSi_sculpin

# Descriptive statistics
round(mean(PSi_trout, na.rm = T), 3)
round(sd(PSi_trout, na.rm = T), 3)
round(mean(PSi_sculpin, na.rm = T), 3)
round(sd(PSi_sculpin, na.rm = T), 3)

# Does PSi_trout differ from PSi_sculpin?
t.test(PSi_trout, PSi_sculpin) # yes, the average diet overlap between stickleback and sculpin is substantially higher than between stickleback and trout


##################################################################################################
## 3) Does trout and/or sculpin competition affect interspecific diet overlap with stickleback? ##
##################################################################################################

# Does competition have an effect on PSi_trout?
model <- lm(PSi_trout ~ Block + Trout*Sculpin + Standard.length, data) # check whether there is a trout:sculpin interaction
Anova(model, type = "III") # drop interaction because it is not significant; re-run reduced model without interaction
model.reduced <- lm(PSi_trout ~ Trout + Sculpin + Standard.length + Block, data) # reduced model
Anova(model.reduced, type = "III") # no clear effect of competiton on trout-stickleback diet overlaps

# Does competition have an effect on PSi_sculpin?
model <- lm(PSi_sculpin ~ Block + Trout*Sculpin + Standard.length, data) # check whether there is a trout:sculpin interaction
Anova(model, type = "III") # drop interaction because it is not significant; re-run reduced model without interaction
model.reduced <- lm(PSi_sculpin ~ Trout + Sculpin + Standard.length + Block, data) # reduced model
Anova(model.reduced, type = "III") # sculpin competition clearly reduces the average sculpin-stickleback diet overlap


###################################################################################
## 3) Does sculpin competition cause stickleback to forage less on benthic prey? ##
###################################################################################

# Calculate bentich prey proportion for each stickleback individual
diet.items.sort.benthic.cats <- match.with.wild$Bolnick.et.al.category[which(match.with.wild$habitat == "benthic")]
diet.items.sort.limnetic.cats <- match.with.wild$Bolnick.et.al.category[which(match.with.wild$habitat == "limnetic")]
limnetic.columns <- match(diet.items.sort.limnetic.cats, names(dietitems.sort))
benthic.columns <- match(diet.items.sort.benthic.cats, names(dietitems.sort))
benth.lim.items.per.fish.t <- data.frame(apply(X = dietitems.sort[, benthic.columns], MAR = 1, FUN = sum, na.rm = T), apply(X = dietitems.sort[, limnetic.columns], MAR = 1, FUN = sum, na.rm = T))
names(benth.lim.items.per.fish.t) <- c("benthic", "limnetic")
benth.lim.items.per.fish.t$totCount <- apply(X = dietitems.sort, MAR = 1, FUN = sum, na.rm = T)
names(benth.lim.items.per.fish.t) <- c("benthic", "limnetic", "totCount")
benth.lim.items.per.fish <- data.frame(data[, c(5:7,9)], benth.lim.items.per.fish.t)
benth.lim.items.per.fish$proportion.benthic.prey <- benth.lim.items.per.fish$benthic / benth.lim.items.per.fish$totCount

# Test whether sculpin and/or trout competition influence benthic foraging in stickleback
model <- lm(proportion.benthic.prey ~ Sculpin + Trout + Block + Standard.length, data = benth.lim.items.per.fish)
Anova(model, type = "III") # sculpin competition makes stickleback forage on less benthic prey


################################################################################
## 4) Which prey categories mainly change in response to sculpin competition? ##
################################################################################

# Calculate the frequency of each prey category in all stickles in cages with sculpin vs. all cages without sculpin
dd <- cbind.data.frame(data[, c("ID", "Lake", "Block", "Sculpin", "Trout")], stickle.dietproportions)
allPrey <- names(dd)[6:dim(dd)[2]]
mean.freq.per.prey.treatment <- aggregate(dd[, allPrey], by = list(dd$Sculpin), FUN = mean)
names(mean.freq.per.prey.treatment)[1] <- "Sculpin"
allPreyMeans <- melt(data.table(mean.freq.per.prey.treatment), id.vars = "Sculpin")
names(allPreyMeans)[2] <- "Bolnick.category"
names(match.with.wild)[1] <- "Bolnick.category"
allPreyMeans <- merge(allPreyMeans, match.with.wild, by = "Bolnick.category", all = TRUE)

# Match data from Bolnick et al. with the data from wild populations to see whether prey shifts are similar
prey <- names(dietitems.sort)
for(i in 1:length(prey)){
       temp <- subset(match.with.wild, match.with.wild$Bolnick.category == prey[i])
       if(length(temp[, 1]) == 1 & !is.na(temp$match.broad.category.of.wild.Gac)){
           names(dietitems.sort)[i] <- as.character(temp$match.broad.category.of.wild.Gac)
       }
       if(length(temp[, 1]) == 1 & is.na(temp$match.broad.category.of.wild.Gac)){
           names(dietitems.sort)[i] <- as.character("no.match")
       }
}

# Collapse columns that now have the same "general" prey category into one
dietitems.sort.collapsed <- t(rowsum(t(dietitems.sort), group = colnames(dietitems.sort)))

# Calculate individual prey proportions
dietitems.sort.collapsed.prop <- dietitems.sort.collapsed/rowSums(dietitems.sort.collapsed)

# These are the 5 most frequent prey categories in the experiemnt and the wild data
select <- c("limnetic.cladoceran", "chironomid", "gammarus", "ephemeroptera.nymph", "trichoptera.larva", "eggs", "chydoridae")
dietitems.sort.collapsed.prop <- as.data.frame(dietitems.sort.collapsed.prop)
dietitems.sort.collapsed.prop <- dietitems.sort.collapsed.prop[, select]

# Prepare data for plotting
stripchartPlot <- cbind.data.frame(data[, c(6)], dietitems.sort.collapsed.prop)
names(stripchartPlot)[1] <- "Sculpin"
stripchartPlot <- aggregate(stripchartPlot[, c(2:dim(stripchartPlot)[2])], by = list(stripchartPlot$Sculpin), FUN = mean)
names(stripchartPlot)[1] <- "Sculpin"
stripchartPlot <- melt(data.table(stripchartPlot), id.vars = "Sculpin")
stripchartPlot$habitat <- match.with.wild$habitat[match(stripchartPlot$variable, match.with.wild$match.broad.category.of.wild.Gac)]
stripchartPlot$habitat <- as.character(stripchartPlot$habitat)
stripchartPlot[is.na(stripchartPlot)] <- as.character("unclear.likely.limnetic")

# Visualize the result
stripchartPlot %>% ggplot(aes(x = Sculpin, y = value, group = variable, color = habitat)) +
    ylim(0, 0.37) +
    ylab("Prey frequency") +
    xlab("Enclosure with sculpin competition?") +
    geom_point(shape = 19, size = 3, stroke = 0) +
    geom_line(lwd = 1) +
    theme_bw()

stripchartPlot %>% ggplot(aes(x = Sculpin, y = value, group = variable, color = variable)) +
    ylim(0, 0.37) +
    ylab("Prey frequency") +
    xlab("Enclosure with sculpin competition?") +
    geom_point(shape = 19, size = 3, stroke = 0) +
    geom_line(lwd = 1) +
    theme_bw()
