#####################################################
#####################################################
### STICKLEBACK STABLE ISOTOPE ANALYSES #############
### Code used for Roesti et al. (Ecology Letters) ###
#####################################################
#####################################################

rm(list = ls(all = TRUE))

# Set path to the downloaded dryad folder containing all codes and data for Roesti et al.
setwd("XXXX")

# Load packages
library(visreg)
library(ggplot2)
library(car)
library(emmeans)
library(lmerTest)

# Load data
environmental.d15N <- read.csv("data/environmental.d15N.data.csv", header = TRUE, stringsAsFactor = FALSE) # lake d15N baselines (measured from mussles, snails, or the mean of both)
gac.popMeans <- read.csv("data/morphology.population.means_stickleback.csv", header = TRUE, stringsAsFactor = FALSE) # population trait means and morphological PC axes as obtained from running the code in 'R.code_morphological.traits.r'
gac.data <- read.csv("data/morphology.data_stickleback.csv", header = TRUE, stringsAsFactor = FALSE) # raw individual-level data, including d15N


########################################################
## 1) Initial data transformation and size correction ##
########################################################

# Subset data to relevant columns and log-transform data
gac.SI.data <- gac.data[, c("ID", "population", "type", "SL", "sex", "d15N")]
gac.SI.data <- gac.SI.data[!is.na(gac.SI.data$d15N),]
names(gac.SI.data)[4] <- "size"
gac.SI.data$size <- log(gac.SI.data$size)
gac.SI.data$d15N <- log(gac.SI.data$d15N)

# Size correction (following Berner 2011) using standard length (SL) for size
sizeCorr = function(input_data, input_size, input_trait, input_grouping){
    xnew = input_data
    xnew[[input_size]] = mean(input_data[[input_size]], na.rm = T)
    z = lm(reformulate(c(input_size, input_grouping), input_trait), data = input_data, na.action = na.exclude)
    res = residuals(z)
    yhat = predict(z,xnew)
    size.corr.values = res + yhat
    input_data[[paste0(input_trait, ".sizeCorr")]] <- size.corr.values
    input_data
}
gac.SI.data <- sizeCorr(gac.SI.data, "size", "d15N", "population")

# Obtain size-corrected d15N population means, restore d15N unit, and merge with d15N lake baselines and foraging trait/PC values
gac.SI.means <- aggregate(gac.SI.data$d15N.sizeCorr, by = list(gac.SI.data$population), FUN = mean, na.rm = TRUE)
names(gac.SI.means) <- c("population", "d15N")
gac.SI.means$d15N <- exp(gac.SI.means$d15N)
gac.data <- merge(environmental.d15N[, c("population", "d15N_mean.mussel.snail")], gac.SI.means, by = "population")
gac.data <- merge(gac.data, gac.popMeans[, c("population", "lake.type", "gape.width", "GR.count", "GR.length", "premaxilla.length", "foraging.pc1")], by = "population")
names(gac.data)[2] <- "d15N.lake.baseline"


#################
## 2) Analyses ##
#################

## 2.1) Testing for a difference in d15N between sympatric and allopatric populations while controlling for variation in baseline d15N among lakes
## This test considers only the 15 populations from lakes with available d15N baseline data

gac.data.complete <- gac.data[which(!is.na(gac.data$d15N.lake.baseline)),]

model <- lm(d15N~d15N.lake.baseline + lake.type, data = gac.data.complete)
Anova(model, type = "III")
stats <- as.data.frame(emmeans(model, "lake.type", weights = "equal"), stringsAsFactors = FALSE)
plotData <- cbind(gac.data.complete, visreg(model, "lake.type")$res[, c(2, 4)])
plotData <- plotData[,-4]

# Visualize the result (Fig. 5c)
p <- ggplot(plotData, aes(x = lake.type, y = visregRes)) +
     ylim(6,12) +
     labs(y = "d15N", x = "Lake Type") +
     geom_jitter(position = position_jitter(0.05), shape = 1, cex = 3, stroke = 1.25) +
     geom_segment(aes(x = 1.25, y = stats[1,5], xend = 1.25, yend = stats[1,6]), colour = "grey", size = 0.75) +
     geom_segment(aes(x = 1.75, y = stats[2,5], xend = 1.75, yend = stats[2,6]), colour = "grey", size = 0.75) +
     geom_point(aes(x = 1.25, y = stats[1,2]), shape = 19, colour = "grey", size = 6) +
     geom_point(aes(x = 1.75, y = stats[2,2]), shape = 19, colour = "grey", size = 6) +
     theme_minimal()
p


## 2.2) Testing whether niche widths of sympatric populations are reduced relative to allopatric populations
## This test considers all populations and uses the variance of size-corrected d15N values within populations as a proxy for niche width

gac.var <- cbind.data.frame(aggregate(exp(gac.SI.data$d15N.sizeCorr), by = list(gac.SI.data$population), FUN = var, na.rm = TRUE), aggregate(exp(gac.SI.data$d15N.sizeCorr), by = list(gac.SI.data$population), FUN = mean, na.rm = TRUE))
gac.var <- gac.var[,-3]
names(gac.var) <- c("popualtion", "var", "mean")
dim(gac.var)
gac.var$lake.type <- as.factor(c("sym", "allo", "sym", "allo", "allo", "allo", "allo", "allo", "sym", "sym", "allo", "sym", "sym", "sym", "sym", "sym", "allo", "allo"))

# Sqrt-transformation normalizes the data well
qqnorm(sqrt(gac.var$var), pch = 1, frame = FALSE, main = "Normal Q-Q plot; sqrt-transformed data")
qqline(sqrt(gac.var$var), col = "steelblue", lwd = 2)
summary(lm(sqrt(var) ~ sqrt(mean), data = gac.var)) # mean does not influence variances, but still add it into model to be consistent with the sculpin model (see below)

# Testing for a niche width difference between population types (mean d15N is added as a co-variate), visualize result (SupplMat Figure)
model <- lm(sqrt(var) ~ lake.type + sqrt(mean), data = gac.var)
Anova(model, type = "III")
p <- visreg(model, "lake.type", xlab = "Population Type", ylab = "Within-population d15N variance", gg = TRUE, points = list(size = 4, shape = 16)) +
     theme_bw()
p


## 2.3) Testing for an association between d15N and foraging trait variation across populations
## These tests consider only the 15 populations from lakes with available baseline d15N estimates; visualize statistically significant associations (SupplMat Figures)

# d15N vs. foraging PC1 (sympatric populations have higher foraging PC1 values than allopatric populations; higher PC1 values = more limnetic foraging morphology)
model <- lm(foraging.pc1 ~ d15N + d15N.lake.baseline, data = gac.data)
Anova(model, type = "III")
p <- visreg(model, "d15N", ylab = "Foraging PC1", ylab = "d15N", gg = TRUE, points = list(size = 3, shape = 16)) +
     theme_bw()
p

# d15N vs. gill raker number
model <- lm(GR.count ~ d15N + d15N.lake.baseline, data = gac.data)
Anova(model, type = "III")
p <- visreg(model, "d15N", ylab = "Gill raker number", ylab = "d15N", gg = TRUE, points = list(size = 3, shape = 16)) +
     theme_bw()
p

# d15N vs. gill raker length
model <- lm(GR.length ~ d15N + d15N.lake.baseline, data = gac.data)
Anova(model, type = "III")
p <- visreg(model, "d15N", ylab = "Gill raker length", ylab = "d15N", gg = TRUE, points = list(size = 3, shape = 16)) +
     theme_bw()
p

# d15N vs. gape width
model <- lm(gape.width ~ d15N + d15N.lake.baseline, data = gac.data)
Anova(model, type = "III")
p <- visreg(model, "d15N", ylab = "Gape width", ylab = "d15N", gg = TRUE, points = list(size = 3, shape = 16)) +
     theme_bw()
p

# d15N vs. premaxialla length
model <- lm(premaxilla.length ~ d15N + d15N.lake.baseline, data = gac.data)
Anova(model, type = "III")


############################################################################################################################################################
############################################################################################################################################################


#####################################################
#####################################################
### SCULPIN STABLE ISOTOPE ANALYSES #################
### Code used for Roesti et al. (Ecology Letters) ###
#####################################################
#####################################################

rm(list = ls(all = TRUE))

# Set path to the downloaded dryad folder containing all codes and data for Roesti et al.
setwd("XXXX")

# Load packages
library(visreg)
library(ggplot2)
library(car)
library(emmeans)
library(lmerTest)

# Load data
environmental.d15N <- read.csv("data/environmental.d15N.data.csv", header = TRUE, stringsAsFactor = FALSE) # lake d15N baselines (measured from mussles, snails, or the mean of both)
cas.popMeans <- read.csv("data/morphology.population.means_sculpin.csv", header = TRUE, stringsAsFactor = FALSE) # population trait means and morphological PC axes as obtained from running the code in 'R.code_morphological.traits.r'
cas.data <- read.csv("data/morphology.data_sculpin.csv", header = TRUE, stringsAsFactor = FALSE) # raw individual-level data, including d15N


########################################################
## 1) Initial data transformation and size correction ##
########################################################

# Subset data to relevant columns and log-transform data
cas.SI.data <- cas.data[, c("ID", "population", "type", "SL", "sex", "d15N")]
cas.SI.data <- cas.SI.data[!is.na(cas.SI.data$d15N),]
names(cas.SI.data)[4] <- "size"
cas.SI.data$size <- log(cas.SI.data$size)
cas.SI.data$d15N <- log(cas.SI.data$d15N)

# Size correction (following Berner 2011) using standard length (SL) for size
sizeCorr = function(input_data, input_size, input_trait, input_grouping){
    xnew = input_data
    xnew[[input_size]] = mean(input_data[[input_size]], na.rm = T)
    z = lm(reformulate(c(input_size, input_grouping), input_trait), data = input_data, na.action = na.exclude)
    res = residuals(z)
    yhat = predict(z,xnew)
    size.corr.values = res + yhat
    input_data[[paste0(input_trait, ".sizeCorr")]] <- size.corr.values
    input_data
}
cas.SI.data <- sizeCorr(cas.SI.data, "size", "d15N", "population")

# Obtain size-corrected d15N population means, restore d15N unit, and merge with d15N lake baselines and foraging trait/PC values
cas.SI.means <- aggregate(cas.SI.data$d15N.sizeCorr, by = list(cas.SI.data$population), FUN = mean, na.rm = TRUE)
names(cas.SI.means) <- c("population", "d15N")
cas.SI.means$d15N <- exp(cas.SI.means$d15N)
cas.data <- merge(environmental.d15N[, c("population", "d15N_mean.mussel.snail")], cas.SI.means, by = "population")
cas.data <- merge(cas.data, cas.popMeans[, c("population", "lake.type", "gape.width", "GR.count", "GR.length", "premaxilla.length", "foraging.pc2")], by = "population")
names(cas.data)[2] <- "d15N.lake.baseline"


#################
## 2) Analyses ##
#################

## 2.1) Testing for a difference in d15N between sympatric and allopatric populations while controlling for variation in baseline d15N among lakes
## This test considers only the 15 populations from lakes with available d15N baseline data

cas.data.complete <- cas.data[which(!is.na(cas.data$d15N.lake.baseline)),]
model <- lm(d15N~d15N.lake.baseline + lake.type, data = cas.data.complete)
Anova(model, type = "III")
stats <- as.data.frame(emmeans(model, "lake.type", weights = "equal"), stringsAsFactors = FALSE)
plotData <- cbind(cas.data.complete, visreg(model, "lake.type")$res[, c(2,4)])
plotData <- plotData[,-4]

# Visualize the result (Fig. 5c)
p <- ggplot(plotData, aes(x = lake.type, y = visregRes)) +
     ylim(6,12) +
     labs(y = "d15N", x = "Lake Type") +
     geom_jitter(position = position_jitter(0.05), shape = 1, cex = 3, stroke = 1.25) +
     geom_segment(aes(x = 1.25, y = stats[1,5], xend = 1.25, yend = stats[1,6]), colour = "grey", size = 0.75) +
     geom_segment(aes(x = 1.75, y = stats[2,5], xend = 1.75, yend = stats[2,6]), colour = "grey", size = 0.75) +
     geom_point(aes(x = 1.25, y = stats[1,2]), shape = 19, colour = "grey", size = 6) +
     geom_point(aes(x = 1.75, y = stats[2,2]), shape = 19, colour = "grey", size = 6) +
     theme_minimal()
p


## 2.2) Testing whether niche widths of sympatric populations are reduced relative to allopatric populations
## This test considers all populations and uses the variance of size-corrected d15N values within populations as a proxy for niche width

cas.var <- cbind.data.frame(aggregate(exp(cas.SI.data$d15N.sizeCorr), by = list(cas.SI.data$population), FUN = var, na.rm = TRUE), aggregate(exp(cas.SI.data$d15N.sizeCorr), by = list(cas.SI.data$population), FUN = mean, na.rm = TRUE))
cas.var <- cas.var[,-3]
names(cas.var) <- c("popualtion", "var", "mean")
dim(cas.var)
cas.var$lake.type <- as.factor(c("sym", "allo", "allo", "sym", "allo", "allo", "allo", "sym", "sym", "sym", "allo", "sym", "sym", "sym", "sym", "allo", "allo"))

# Log-transformation normalizes the data well
qqnorm(log(cas.var$var), pch = 1, frame = FALSE, main = "Normal Q-Q plot; log-transformed data")
qqline(log(cas.var$var), col = "steelblue", lwd = 2)
summary(lm(log(var) ~ log(mean), data = cas.var)) # mean influences the variance, thus add it as a covariate into the model

# Testing for a niche width difference between population types (mean d15N is added as a co-variate), visualize result (SupplMat Figure)
model <- lm(log(var) ~ lake.type + log(mean), data = cas.var)
Anova(model, type = "III")
p <- visreg(model, "lake.type", xlab = "Population Type", ylab = "Within-population d15N variance", gg = TRUE, points = list(size = 4, shape = 16)) +
     theme_bw()
p


## 2.3) Testing for an association between d15N and foraging trait variation across populations
## These tests consider only the 15 populations from lakes with available baseline d15N estimates; visualize statistically significant association (SupplMat Figure)

# d15N vs. foraging PC2 (allopatric and sympatric sculpin populations differ along foraging PC2)
model <- lm(foraging.pc2 ~ d15N + d15N.lake.baseline, data = cas.data)
Anova(model, type = "III")

# d15N vs. gill raker number
model <- lm(GR.count ~ d15N + d15N.lake.baseline, data = cas.data)
Anova(model, type = "III")
p <- visreg(model, "d15N", ylab = "Gill raker number", ylab = "d15N", gg = TRUE, points = list(size = 3, shape = 16)) +
     theme_bw()
p

# d15N vs. gill raker length
model <- lm(GR.length ~ d15N + d15N.lake.baseline, data = cas.data)
Anova(model, type = "III")

# d15N vs. gape width
model <- lm(gape.width ~ d15N + d15N.lake.baseline, data = cas.data)
Anova(model, type = "III")

# d15N vs. premaxialla length
model <- lm(premaxilla.length ~ d15N + d15N.lake.baseline, data = cas.data)
Anova(model, type = "III")
