#####################################################
#####################################################
### DIET (STOMACH CONTENT) ANALYSES #################
### Code used for Roesti et al. (Ecology Letters) ###
#####################################################
#####################################################

rm(list = ls(all = TRUE))

# Set path to the downloaded dryad folder containing all codes and data for Roesti et al.
setwd("XXXX")

# Load packages
library(dplyr)
library(data.table)
library(vegan)
library(ggplot2)
library(car)

# Load data for both species
gac.ind.diets <- read.csv("data/stomach.contents_stickleback.csv", header = TRUE, stringsAsFactor = FALSE) # stomach content data of stickleback (excluding individuals with empty stomachs)
cas.ind.diets <- read.csv("data/stomach.contents_sculpin.csv", header = TRUE, stringsAsFactor = FALSE) # stomach content data of sculpin (excluding individuals with empty stomachs)
gac.diet.categ <- read.csv("data/diet.categories_stickleback.csv", header = TRUE, stringsAsFactor = FALSE) # diet categories for stickleback
cas.diet.categ <- read.csv("data/diet.categories_sculpin.csv", header = TRUE, stringsAsFactor = FALSE) # diet categories for sculpin


#######################################################################
## 1) Initial data transformation, calculate some summary statistics ##
#######################################################################

## Stickleback

gac.info <- gac.ind.diets[,c(1:5)]
gac.ind.diets <- gac.ind.diets[,-c(1:5)]

# Remap detailed diet categories to more general categories, then collapse data from the same general categories
gac.ind.diets <- gac.ind.diets[, match(gac.diet.categ$specific, names(gac.ind.diets))]
names(gac.ind.diets) <- gac.diet.categ[,2]
gac.ind.diets <- cbind.data.frame(ID = gac.info[,1], population = gac.info[,2], lake.type = gac.info[,3], SL = gac.info[,4],
                            sapply(unique(colnames(gac.ind.diets)),
                            function(x){rowSums(gac.ind.diets[, grep(x, colnames(gac.ind.diets)), drop = FALSE])}))

# Prey proportions within individuals
gac.ind.diet.prop <- cbind.data.frame(gac.ind.diets[,c(1:4)], (gac.ind.diets[,-c(1:4)]/rowSums(gac.ind.diets[,-c(1:4)])))

# Sample sizes
round(mean(table(gac.ind.diets$population)),1) # average per population
dim(gac.ind.diets)[1] # total

# Population-level diets and grand means per lake type
gac.pop.diets <- as.data.table(gac.ind.diet.prop[,-c(1,3,4)])
gac.pop.diets <- gac.pop.diets[, lapply(.SD, mean), by = population]
lake.type <- c("sym", "sym", "sym", "allo", "allo", "allo", "allo", "sym", "allo", "sym", "sym", "sym", "allo", "sym", "allo", "allo", "sym", "allo")
gac.pop.diets <- setDT(cbind(gac.pop.diets, lake.type))
gac.pop.grand.means <- suppressWarnings(data.frame(t(aggregate(as.data.frame(gac.pop.diets[,-1]), by = list(gac.pop.diets$lake.type), FUN = mean)))[-(dim(gac.pop.diets)[2]), ])
gac.pop.grand.means


## Sculpin

cas.info <- cas.ind.diets[,c(1:5)]
cas.ind.diets <- cas.ind.diets[,-c(1:5)]

# Remap detailed diet categories to more general categories, then collapse data from the same general categories
cas.ind.diets <- cas.ind.diets[, match(cas.diet.categ$specific, names(cas.ind.diets))]
names(cas.ind.diets) <- cas.diet.categ[,2]
cas.ind.diets <- cbind.data.frame(ID = cas.info[,1], population = cas.info[,2], lake.type = cas.info[,3], SL = cas.info[,4],
                            sapply(unique(colnames(cas.ind.diets)),
                            function(x){rowSums(cas.ind.diets[, grep(x, colnames(cas.ind.diets)), drop = FALSE])}))

# Prey proportions within individuals
cas.ind.diet.prop <- cbind.data.frame(cas.ind.diets[,c(1:4)], (cas.ind.diets[,-c(1:4)]/rowSums(cas.ind.diets[,-c(1:4)])))

# Sample sizes
round(mean(table(cas.ind.diets$population)),1) # average per population
dim(cas.ind.diets)[1] # total

# Population-level diets and grand means per lake type
cas.pop.diets <- as.data.table(cas.ind.diet.prop[,-c(1,3,4)])
cas.pop.diets <- cas.pop.diets[, lapply(.SD, mean), by = population]
lake.type <- c("allo", "allo", "allo", "sym", "allo", "sym", "sym", "allo", "allo", "allo", "sym", "sym", "sym", "sym", "sym", "allo", "sym")
cas.pop.diets <- setDT(cbind(cas.pop.diets, lake.type))
cas.pop.grand.means <- suppressWarnings(data.frame(t(aggregate(as.data.frame(cas.pop.diets[,-1]), by = list(cas.pop.diets$lake.type), FUN = mean)))[-(dim(cas.pop.diets)[2]),])
cas.pop.grand.means


###################################
## 2) Interspecific diet overlap ##
###################################

## Percent of shared prey categories between species

# Remove "unknown.invertebrate" category as this is not informative for diet overlaps
gac.pop.diets.safe <- gac.pop.diets # will be used later for the niche shift analysis
cas.pop.diets.safe <- cas.pop.diets # will be used later for the niche shift analysis
gac.pop.diets <- gac.pop.diets[, -c("unknown.invertebrate", "lake.type")]
cas.pop.diets <- cas.pop.diets[, -c("unknown.invertebrate", "lake.type")]
gac.prey <- colnames(gac.pop.diets)[-1]
cas.prey <- colnames(cas.pop.diets)[-1]
round(length(intersect(gac.prey, cas.prey))/length(gac.prey), 2)*100
round(length(intersect(gac.prey, cas.prey))/length(cas.prey), 2)*100


## Prepare data for interspecific diet overlap analysis

# Define Schoener diet overlap function and sympatric lakes
schoener <- function(x, y){sum(pmin(x, y))}
symp.lakes <- c("Ambrose", "Blackwater", "McNair", "Merrill", "Morton", "North", "Pachena", "Paq", "Roselle")

# Add empty columns to the data frame of each species for non-shared prey categories
unique.gac <- setdiff(colnames(gac.pop.diets), colnames(cas.pop.diets))
unique.cas <- setdiff(colnames(cas.pop.diets), colnames(gac.pop.diets))
gac.pop.diets[, unique.cas] <- 0
cas.pop.diets[, unique.gac] <- 0

# Match column order between species and filter for prey categories present in at least one population of each lake type
selectCols <- colnames(gac.pop.diets)
cas.pop.diets <- cas.pop.diets[, ..selectCols]
unique(names(cas.pop.diets) == names(gac.pop.diets)) # should yield only 'TRUE'

g1 <- gac.pop.diets[population %in% symp.lakes, colSums(.SD), .SDcols = !c("population")] %>% .[.>0] %>% names(.)
g2 <- gac.pop.diets[!population %in% symp.lakes, colSums(.SD), .SDcols = !c("population")] %>% .[.>0] %>% names(.)
c1 <- cas.pop.diets[population %in% symp.lakes, colSums(.SD), .SDcols = !c("population")] %>% .[.>0] %>% names(.)
c2 <- cas.pop.diets[!population %in% symp.lakes, colSums(.SD), .SDcols = !c("population")] %>% .[.>0] %>% names(.)
restrictedPrey <- intersect(intersect(g1, g2), intersect(c1, c2))
restrictedPrey <- restrictedPrey[!restrictedPrey %in% c("eggs", "unknown.invertebrate")] # remove "eggs" and "unknown.invertebrate"


## Calculate interspecific diet overlaps and visualize result

# Calculate symp-symp and allo-allo overlaps
gac.cas.pairs <- expand.grid(unique(gac.pop.diets$population), unique(cas.pop.diets$population))

sym.sym.pairs <- gac.cas.pairs[gac.cas.pairs[,1] %in% symp.lakes & gac.cas.pairs[,2] %in% symp.lakes, ]
dim(sym.sym.pairs) # number of pairwise symp-symp interspecific population comparisons
sym.sym.schoener <- apply(sym.sym.pairs, MARGIN = 1, FUN = function(z) schoener(gac.pop.diets[population == z[1], ..restrictedPrey],
                                                                           cas.pop.diets[population == z[2], ..restrictedPrey]))
allo.allo.pairs <- gac.cas.pairs[!gac.cas.pairs[, 1] %in% symp.lakes & !gac.cas.pairs[,2] %in% symp.lakes, ]
dim(allo.allo.pairs) # number of pairwise allo-allo interspecific population comparisons
allo.allo.schoener <- apply(allo.allo.pairs, MARGIN = 1, FUN = function(z) schoener(gac.pop.diets[population == z[1], ..restrictedPrey],
                                                                         cas.pop.diets[population == z[2], ..restrictedPrey]))
# Visualize interspecific diet overlaps (Figure 5a)
comparison <- c(rep("sym-sym", length(sym.sym.schoener)), rep("allo-allo", length(allo.allo.schoener)))
ovlPlotData <- data.table(overlap = c(sym.sym.schoener, allo.allo.schoener), comparison = comparison)
p <- ggplot(ovlPlotData, aes(x = comparison, y = overlap)) +
     ylim(0, 0.62) +
     ylab("Diet overlap") +
     geom_boxplot() +
     geom_jitter(shape = 20, position = position_jitter(0.1), alpha = 0.4, size = 2.5) +
     theme_minimal()
p


## Testing the difference of interspecific diet overlaps between lake types using permutation

iter <- 10000 # number of resampling iterations; note that running 10,000 iterations on a desktop computer takes at least 2h

sym.mean.obs <- mean(sym.sym.schoener)
allo.mean.obs <- mean(allo.allo.schoener)
abs.diff.obs <- abs(sym.mean.obs-allo.mean.obs)

gac.pop.diets.perm <- gac.pop.diets
cas.pop.diets.perm <- cas.pop.diets

cnt <- 0
for(i in 1:iter){
    gac.pop.diets.perm[, "population"] <- sample(gac.pop.diets$population)
    cas.pop.diets.perm[, "population"] <- sample(cas.pop.diets$population)
    sym.sym.schoenerPerm <- apply(sym.sym.pairs, MARGIN = 1, FUN = function(z) schoener(gac.pop.diets.perm[population == z[1], ..restrictedPrey], cas.pop.diets.perm[population == z[2], ..restrictedPrey]))
    allo.allo.schoenerPerm <- apply(allo.allo.pairs, MARGIN = 1, FUN = function(z) schoener(gac.pop.diets.perm[population == z[1], ..restrictedPrey], cas.pop.diets.perm[population == z[2], ..restrictedPrey]))
    abs.diff.perm <- abs(mean(sym.sym.schoenerPerm)-mean(allo.allo.schoenerPerm))
    if(abs.diff.perm >= abs.diff.obs){cnt <- cnt+1}
}

Pvalue <- round(as.numeric(cnt/iter), digits = 3)
Pvalue


########################################################
## 3) Unconstrained ordination including both species ##
########################################################

# Combine data sets with both lake types, append species identifier to lake names
gac.pop.diets[, species := "stickleback"]
cas.pop.diets[, species := "sculpin"]
all.pop.diets <- rbind(gac.pop.diets[, c("population", "species", restrictedPrey), with = F],
                      cas.pop.diets[, c("population", "species", restrictedPrey), with = F])
all.pop.diets <- data.frame(all.pop.diets[, -c("population", "species")], row.names = all.pop.diets[, paste(population, species, sep = "_")])

# Call the unconstrained ordination
dietCCA <- cca(all.pop.diets)
scrs <- scores(dietCCA, display = c("sites", "species"))
summary(dietCCA) # how much variaton does CA1 explain?
dfSites <- data.table(scrs$sites,t(as.data.frame(strsplit(rownames(scrs$sites),"_"))))
setnames(dfSites, c("V1","V2"), c("population", "species"))
dfSites[population %in% symp.lakes, lake.type := "sym"]
dfSites[!population %in% symp.lakes, lake.type := "allo"]
plotCA <- cbind.data.frame(dfSites$species, dfSites$population, dfSites$lake.type, dfSites$CA1)
names(plotCA) <- c("species", "population", "lake.type", "CA1")

# Calculatate non-parametric 95% CI for plotting (takes 1-2 minutes to run on a desktop computer)
plotCA$inf <- paste0(plotCA$species, "_", plotCA$lake.type)
inf2 <- unique(plotCA$inf)
plotCA$CA1 <- plotCA$CA1*(-1) # reverse the arbitrary orientation of CA1 to be more intuitive

alp <- 0.05 # alpha-level
iter <- 10000 # number of resampling iterations

data <- NULL
for(o in 1:length(inf2)){
    d1 <- subset(plotCA, plotCA$inf == inf2[o])$CA1
    rdist <- NULL
    for(i in 1:iter){
        rdist <- c(rdist, mean(sample(d1, length(d1), replace = T)))
        }
    mean.CA1 <- mean(d1)
    CA1.CI.low <- sort(rdist)[alp/2*iter+1]
    CA1.CI.up <- sort(rdist)[(1-(alp/2))*iter]

    out <- cbind(inf2[o], mean.CA1, CA1.CI.low, CA1.CI.up)
    data <- rbind(out, data)
}
dataCI <- as.data.frame(data)
names(dataCI)[1] <- "type"
dataCI[,2] <- as.numeric(as.character(dataCI[,2])) # make numeric
dataCI[,3] <- as.numeric(as.character(dataCI[,3])) # make numeric
dataCI[,4] <- as.numeric(as.character(dataCI[,4])) # make numeric

# Visualize result (Figure 5b)
level_order <- c("stickleback_allo", "stickleback_sym", "sculpin_allo", "sculpin_sym")
p <- ggplot(plotCA, aes(y = CA1, x = factor(inf, level = level_order))) +
     theme_classic() +
     geom_jitter(position = position_jitter(0.1), shape = 1, cex = 2.25, stroke = 1.1) +
     geom_point(x = 1.25, y = dataCI[3,2], size = 6, shape = 16) +
     geom_point(x = 1.75, y = dataCI[4,2], size = 6, shape = 16) +
     geom_point(x = 3.75, y = dataCI[1,2], size = 6, shape = 16) +
     geom_point(x = 3.25, y = dataCI[2,2], size = 6, shape = 16) +
     # Add non-parametric 955CI bars
     geom_segment(aes(x = 1.25, y = dataCI[3,3], xend = 1.25, yend = dataCI[3,4]), size = 0.3) +
     geom_segment(aes(x = 1.75, y = dataCI[4,3], xend = 1.75, yend = dataCI[4,4]), size = 0.3) +
     geom_segment(aes(x = 3.75, y = dataCI[1,3], xend = 3.75, yend = dataCI[1,4]), size = 0.3) +
     geom_segment(aes(x = 3.25, y = dataCI[2,3], xend = 3.25, yend = dataCI[2,4]), size = 0.3) +
     ylab("Diet CA1 (30.2%)") +
     xlab("Population type")
p


##################################################################
## 3) Testing for a benthic-limnetic niche shift in stickleback ##
##################################################################

## Preparing data for analysis

# Make data tables
setDT(gac.diet.categ)
setDT(gac.ind.diets)
setDT(gac.pop.diets)

# Identify limnetic and benthic prey categories and calculate the mean benthic prey proportion per individual
benthicPrey <- gac.diet.categ[habitat == "benthic", unique(general)]
allPrey <- names(gac.ind.diets)[!names(gac.ind.diets) %in% c("ID", "population", "lake.type", "SL")]
allCount <- gac.ind.diets[, rowSums(.SD), .SDcols = allPrey]
benthicCount <- gac.ind.diets[, rowSums(.SD), .SDcols = benthicPrey]
totalCount <- gac.ind.diets[, rowSums(.SD), .SDcols = allPrey]
gac.ind.diets[, benthProp := benthicCount/totalCount]

# Size correction (following Berner 2011) using standard length (SL) for size because there is a clear effect of size on benthic prey proportions within populations
Anova(lm(benthProp ~ population + SL, data = gac.ind.diets), type = "III")

sizeCorr = function(input_data, input_size, input_trait, input_grouping){
    xnew = input_data
    xnew[[input_size]] = mean(input_data[[input_size]], na.rm = T)
    z = lm(reformulate(c(input_size, input_grouping), input_trait), data = input_data, na.action = na.exclude)
    res = residuals(z)
    yhat = predict(z,xnew)
    size.corr.values = res + yhat
    input_data[[paste0(input_trait, ".sizeCorr")]] <- size.corr.values
    input_data
}
gac.ind.diets <- sizeCorr(gac.ind.diets, "SL", "benthProp", "population")
popMeans <- data.frame(aggregate(gac.ind.diets$benthProp.sizeCorr, by = list(gac.ind.diets$population, gac.ind.diets$lake.type), FUN = mean, na.rm = TRUE))
names(popMeans)[1] <- "population"
names(popMeans)[2] <- "type"
names(popMeans)[3] <- "mean.benthicProportion"


## Testing for a difference in the proportion of benthic prey between lake types: calculate the non-parametric P-value and 95% CI

# Observed grand mean of benthic prey proportion for each lake type
obs.mean.allo <- mean(subset(popMeans, popMeans$type == "allo")[,3])
obs.mean.sym <- mean(subset(popMeans, popMeans$type == "sym")[,3])

# Set parameters for calculating non-parametric P-values and 95% CI
alp <- 0.05 # alpha-level
iter <- 10000 # number of resampling iterations

# P-value
cnt <- 0
for(i in 1:iter){
    x <- popMeans
    x[,2] <- popMeans[sample(1:length(popMeans[,2])),2]
    allo.mean <- mean(subset(x, x$type == "allo")[, "mean.benthicProportion"])
    sym.mean <- mean(subset(x, x$type == "sym")[, "mean.benthicProportion"])
    sim.mean.diff <- allo.mean-sym.mean
    if(abs(sim.mean.diff)>= abs(obs.mean.allo - obs.mean.sym)){cnt <- cnt+1}
}
p.value <- round(as.numeric((cnt)/(iter)), digits = 3)
p.value

# Sympatric 95% CI
d <- popMeans[which(popMeans$type == "sym"),3]
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.sym <- sort(rdist)[alp/2*iter+1]
upper.CI.sym <- sort(rdist)[(1-(alp/2))*iter]

# Allopatric 95% CI
d <- popMeans[which(popMeans$type == "allo"),3]
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.allo <- sort(rdist)[alp/2*iter+1]
upper.CI.allo <- sort(rdist)[(1-(alp/2))*iter]


## Visualize the difference in the proportion of benthic prey between stickleback populations from different lake types (Figure 5d)

p <- ggplot(popMeans, aes(x = type, y = mean.benthicProportion)) +
     ylim(0, 1) +
     labs(y = "Proportion benthic prey", x = "Lake Type") +
     geom_jitter(position = position_jitter(0.05), shape = 1, cex = 3, stroke = 1.25) +
     geom_segment(aes(x = 1.25, y = low.CI.allo, xend = 1.25, yend = upper.CI.allo), colour = "grey", size = 0.75) +
     geom_segment(aes(x = 1.75, y = low.CI.sym, xend = 1.75, yend = upper.CI.sym), colour = "grey", size = 0.75) +
     geom_point(aes(x = 1.25, y = obs.mean.allo), shape = 19, colour = "grey", size = 6) +
     geom_point(aes(x = 1.75, y = obs.mean.sym), shape = 19, colour = "grey", size = 6) +
     theme_minimal()
p


##############################################################
## 4) Testing for a benthic-limnetic niche shift in sculpin ##
##############################################################

## Preparing data for analysis

# Make data tables
setDT(cas.diet.categ)
setDT(cas.ind.diets)
setDT(cas.pop.diets)

# Identify limnetic and benthic prey categories and calculate the mean benthic prey proportion per individual
benthicPrey <- cas.diet.categ[habitat == "benthic", unique(general)]
allPrey <- names(cas.ind.diets)[!names(cas.ind.diets) %in% c("ID", "population", "lake.type", "SL")]
allCount <- cas.ind.diets[, rowSums(.SD), .SDcols = allPrey]
benthicCount <- cas.ind.diets[, rowSums(.SD), .SDcols = benthicPrey]
totalCount <- cas.ind.diets[, rowSums(.SD), .SDcols = allPrey]
cas.ind.diets[, benthProp := benthicCount/totalCount]

# Size correction (following Berner 2011) using standard length (SL) for size because there is a clear effect of size on benthic prey proportions within populations
Anova(lm(benthProp ~ population + SL, data = cas.ind.diets), type = "III")

sizeCorr = function(input_data, input_size, input_trait, input_grouping){
    xnew = input_data
    xnew[[input_size]] = mean(input_data[[input_size]], na.rm = T)
    z = lm(reformulate(c(input_size, input_grouping), input_trait), data = input_data, na.action = na.exclude)
    res = residuals(z)
    yhat = predict(z,xnew)
    size.corr.values = res + yhat
    input_data[[paste0(input_trait, ".sizeCorr")]] <- size.corr.values
    input_data
}
cas.ind.diets <- sizeCorr(cas.ind.diets, "SL", "benthProp", "population")
popMeans <- data.frame(aggregate(cas.ind.diets$benthProp.sizeCorr, by = list(cas.ind.diets$population, cas.ind.diets$lake.type), FUN = mean, na.rm = TRUE))
names(popMeans)[1] <- "population"
names(popMeans)[2] <- "type"
names(popMeans)[3] <- "mean.benthicProportion"


## Testing for a difference in the proportion of benthic prey between lake types: calculate the non-parametric P-value and 95% CI

# Observed grand mean of benthic prey proportion for each lake type
obs.mean.allo <- mean(subset(popMeans, popMeans$type == "allo")[,3])
obs.mean.sym <- mean(subset(popMeans, popMeans$type == "sym")[,3])

# Set parameters for calculating non-parametric P-value and 95% CI
alp <- 0.05 # alpha-level
iter <- 10000 # number of resampling iterations

# P-value
cnt <- 0
for(i in 1:iter){
    x <- popMeans
    x[,2] <- popMeans[sample(1:length(popMeans[,2])),2]
    allo.mean <- mean(subset(x, x$type == "allo")[, "mean.benthicProportion"])
    sym.mean <- mean(subset(x, x$type == "sym")[, "mean.benthicProportion"])
    sim.mean.diff <- allo.mean-sym.mean
    if(abs(sim.mean.diff)>= abs(obs.mean.allo - obs.mean.sym)){cnt <- cnt+1}
}
p.value <- round(as.numeric((cnt)/(iter)), digits = 3)
p.value

# Sympatric 95% CI
d <- popMeans[which(popMeans$type == "sym"),3]
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.sym <- sort(rdist)[alp/2*iter+1]
upper.CI.sym <- sort(rdist)[(1-(alp/2))*iter]

# Allopatric 95% CI
d <- popMeans[which(popMeans$type == "allo"),3]
rdist <- rep(9, iter)
for(i in 1:iter){
    rdist[i] <- mean(sample(d, length(d), replace = T))
}
low.CI.allo <- sort(rdist)[alp/2*iter+1]
upper.CI.allo <- sort(rdist)[(1-(alp/2))*iter]


## Visualize the difference in the proportion of benthic prey between sculpin populations from different lake types (Figure 5d)

p <- ggplot(popMeans, aes(x = type, y = mean.benthicProportion)) +
     ylim(0, 1) +
     labs(y = "Proportion benthic prey", x = "Lake Type") +
     geom_jitter(position = position_jitter(0.05), shape = 1, cex = 3, stroke = 1.25) +
     geom_segment(aes(x = 1.25, y = low.CI.allo, xend = 1.25, yend = upper.CI.allo), colour = "grey", size = 0.75) +
     geom_segment(aes(x = 1.75, y = low.CI.sym, xend = 1.75, yend = upper.CI.sym), colour = "grey", size = 0.75) +
     geom_point(aes(x = 1.25, y = obs.mean.allo), shape = 19, colour = "grey", size = 6) +
     geom_point(aes(x = 1.75, y = obs.mean.sym), shape = 19, colour = "grey", size = 6) +
     theme_minimal()
p
